@@ -1575,9 +1575,11 @@ extern void		*malloc();
 #  if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,0,36)) && !defined(HASFCHMOD)
 #    define HASFCHMOD	1	/* fchmod(2) */
 #  endif
+#  if (__GLIBC__ >= 2 && __GLIBC_MINOR__ >= 19) && !defined(HAS_GETHOSTBYNAME2)
+#   define HAS_GETHOSTBYNAME2 1
+#  endif
 # endif /* __linux__ */
 
-
 /*
 **  DELL SVR4 Issue 2.2, and others
 **	From Kimmo Suominen <kim@grendel.lut.fi>
