@@ -6,7 +6,7 @@
  * forth in the LICENSE file which can be found at the top level of
  * the sendmail distribution.
  *
- *	$Id: gen.h,v 1.24 2013/11/22 20:51:31 ca Exp $
+ *	$Id: gen.h,v 1.24 2013-11-22 20:51:31 ca Exp $
  */
 
 /*
@@ -43,7 +43,7 @@
 # else /* SM_CONF_STDDEF_H */
 #  ifndef NULL
 #   define NULL	0
-#  endif /* ! NULL */
+#  endif
 #  define offsetof(type, member)	((size_t)(&((type *)0)->member))
 # endif /* SM_CONF_STDDEF_H */
 
@@ -77,4 +77,16 @@
 typedef int SM_ATOMIC_INT_T;
 typedef unsigned int SM_ATOMIC_UINT_T;
 
+#if _FFR_EAI && !defined(USE_EAI)
+# define USE_EAI 1
+#endif
+
+#if USE_EAI && !defined(_FFR_LOGASIS)
+# define _FFR_LOGASIS 1
+#endif
+
+#if USE_EAI || _FFR_EIGHT_BIT_ADDR_OK
+# define _FFR_8BITENVADDR 1
+#endif
+
 #endif /* SM_GEN_H */
