@@ -21,23 +21,21 @@ SM_IDSTR(copyright,
 SM_IDSTR(id, "@(#)$Id: mail.local.c,v 8.257 2013-11-22 20:51:51 ca Exp $")
 
 #include <stdlib.h>
+#include <sm/sendmail.h>
 #include <sm/errstring.h>
 #include <sm/io.h>
 #include <sm/limits.h>
-# include <unistd.h>
-# ifdef EX_OK
-#  undef EX_OK		/* unistd.h may have another use for this */
-# endif /* EX_OK */
-# define LOCKFILE_PMODE 0
+#include <unistd.h>
+#define LOCKFILE_PMODE 0
 #include <sm/mbdb.h>
 #include <sm/sysexits.h>
 
 #ifndef HASHSPOOL
 # define HASHSPOOL	0
-#endif /* ! HASHSPOOL */
+#endif
 #ifndef HASHSPOOLMD5
 # define HASHSPOOLMD5	0
-#endif /* ! HASHSPOOLMD5 */
+#endif
 
 /*
 **  This is not intended to work on System V derived systems
@@ -76,7 +74,7 @@ SM_IDSTR(id, "@(#)$Id: mail.local.c,v 8.257 2013-11-22 20:51:51 ca Exp $")
 # if HASHSPOOLMD5
 #  define HASH_MD5	2
 #  include <openssl/md5.h>
-# endif /* HASHSPOOLMD5 */
+# endif
 #endif /* HASHSPOOL */
 
 #if _FFR_SPOOL_PATH
@@ -91,10 +89,10 @@ SM_IDSTR(id, "@(#)$Id: mail.local.c,v 8.257 2013-11-22 20:51:51 ca Exp $")
 
 #ifndef LOCKTO_RM
 # define LOCKTO_RM	300	/* timeout for stale lockfile removal */
-#endif /* ! LOCKTO_RM */
+#endif
 #ifndef LOCKTO_GLOB
 # define LOCKTO_GLOB	400	/* global timeout for lockfile creation */
-#endif /* ! LOCKTO_GLOB */
+#endif
 
 /* define a realloc() which works for NULL pointers */
 #define REALLOC(ptr, size)	(((ptr) == NULL) ? malloc(size) : realloc(ptr, size))
@@ -107,13 +105,13 @@ SM_IDSTR(id, "@(#)$Id: mail.local.c,v 8.257 2013-11-22 20:51:51 ca Exp $")
 # define flock(a, b)	lockf(a, b, 0)
 # ifdef LOCK_EX
 #  undef LOCK_EX
-# endif /* LOCK_EX */
+# endif
 # define LOCK_EX	F_LOCK
 #endif /* LDA_USE_LOCKF */
 
 #ifndef LOCK_EX
 # include <sys/file.h>
-#endif /* ! LOCK_EX */
+#endif
 
 /*
 **  If you don't have setreuid, and you have saved uids, and you have
@@ -123,31 +121,34 @@ SM_IDSTR(id, "@(#)$Id: mail.local.c,v 8.257 2013-11-22 20:51:51 ca Exp $")
 
 #ifdef LDA_USE_SETEUID
 # define setreuid(r, e)		seteuid(e)
-#endif /* LDA_USE_SETEUID */
+#endif
 
 #ifdef LDA_CONTENTLENGTH
 # define CONTENTLENGTH	1
-#endif /* LDA_CONTENTLENGTH */
+#endif
 
 #ifndef INADDRSZ
 # define INADDRSZ	4		/* size of an IPv4 address in bytes */
-#endif /* ! INADDRSZ */
+#endif
 
 #ifdef MAILLOCK
 # include <maillock.h>
-#endif /* MAILLOCK */
+#endif
 
 #ifndef MAILER_DAEMON
 # define MAILER_DAEMON	"MAILER-DAEMON"
-#endif /* ! MAILER_DAEMON */
+#endif
 
 #ifdef CONTENTLENGTH
 char	ContentHdr[40] = "Content-Length: ";
 off_t	HeaderLength;
 off_t	BodyLength;
-#endif /* CONTENTLENGTH */
+#endif
 
 bool	EightBitMime = true;		/* advertise 8BITMIME in LMTP */
+#if USE_EAI
+bool	EAI = true;			/* advertise SMTPUTF8 in LMTP */
+#endif
 char	ErrBuf[10240];			/* error buffer */
 int	ExitVal = EX_OK;		/* sysexits.h error value. */
 bool	HoldErrs = false;		/* Hold errors in ErrBuf */
@@ -160,9 +161,9 @@ char	*HomeMailFile = NULL;		/* store mail in homedir */
 int	HashType = HASH_NONE;
 int	HashDepth = 0;
 bool	StripRcptDomain = true;
-#else /* HASHSPOOL */
+#else
 # define StripRcptDomain true
-#endif /* HASHSPOOL */
+#endif
 char	SpoolPath[MAXPATHLEN];
 
 char	*parseaddr __P((char *, bool));
@@ -179,7 +180,7 @@ void	mailerr __P((const char *, const char *, ...));
 void	flush_error __P((void));
 #if HASHSPOOL
 const char	*hashname __P((char *));
-#endif /* HASHSPOOL */
+#endif
 
 
 static void sm_exit __P((int));
@@ -218,11 +219,11 @@ main(argc, argv)
 	/* use a reasonable umask */
 	(void) umask(0077);
 
-# ifdef LOG_MAIL
+#ifdef LOG_MAIL
 	openlog("mail.local", 0, LOG_MAIL);
-# else /* LOG_MAIL */
+#else
 	openlog("mail.local", 0);
-# endif /* LOG_MAIL */
+#endif
 
 	from = NULL;
 
@@ -233,14 +234,16 @@ main(argc, argv)
 		mailerr("421", "Configuration error: _PATH_MAILDIR too large");
 		sm_exit(EX_CONFIG);
 	}
+
+	/* HACK: add U to all options - this should be only for USE_EAI */
 #if HASHSPOOL
-	while ((ch = getopt(argc, argv, "7bdD:f:h:r:lH:p:n")) != -1)
+	while ((ch = getopt(argc, argv, "7bdD:f:h:r:lH:p:nUV")) != -1)
 #else /* HASHSPOOL */
-#  if _FFR_SPOOL_PATH
-	while ((ch = getopt(argc, argv, "7bdD:f:h:r:lp:")) != -1)
-#  else /* _FFR_SPOOL_PATH */
-	while ((ch = getopt(argc, argv, "7bdD:f:h:r:l")) != -1)
-#  endif /* _FFR_SPOOL_PATH */
+# if _FFR_SPOOL_PATH
+	while ((ch = getopt(argc, argv, "7bdD:f:h:r:lp:UV")) != -1)
+# else
+	while ((ch = getopt(argc, argv, "7bdD:f:h:r:lUV")) != -1)
+# endif
 #endif /* HASHSPOOL */
 	{
 		switch(ch)
@@ -302,7 +305,7 @@ main(argc, argv)
 			  case 'm':
 				HashType = HASH_MD5;
 				break;
-# endif /* HASHSPOOLMD5 */
+# endif
 
 			  default:
 				mailerr(NULL, "-H: unknown hash type");
@@ -342,6 +345,22 @@ main(argc, argv)
 			break;
 #endif /* HASHSPOOL || _FFR_SPOOL_PATH */
 
+#if USE_EAI
+		  case 'U':
+			EAI = false;
+			break;
+#endif
+		  case 'V':
+			fprintf(stderr, "compiled with\n");
+#if MAIL_LOCAL_TEST
+			fprintf(stderr, "MAIL_LOCAL_TEST\n");
+#endif
+#if USE_EAI
+			/* test scripts should look for SMTPUTF8 */
+			fprintf(stderr, "USE_EAI\n");
+#endif
+			break;
+
 		  case '?':
 		  default:
 			usage();
@@ -381,7 +400,7 @@ main(argc, argv)
 	}
 
 	/* Non-LMTP from here on out */
-	if (*argv == '\0')
+	if (*argv == NULL)
 		usage();
 
 	/*
@@ -573,7 +592,7 @@ dolmtp()
 		{
 		  case 'd':
 		  case 'D':
-			if (sm_strcasecmp(buf, "data") == 0)
+			if (SM_STRCASEEQ(buf, "data"))
 			{
 				bool inbody = false;
 
@@ -627,6 +646,10 @@ dolmtp()
 				printf("250-%s\r\n", myhostname);
 				if (EightBitMime)
 					printf("250-8BITMIME\r\n");
+#if USE_EAI
+				if (EAI)
+					printf("250-SMTPUTF8\r\n");
+#endif
 				printf("250-ENHANCEDSTATUSCODES\r\n");
 				printf("250 PIPELINING\r\n");
 				continue;
@@ -662,7 +685,7 @@ dolmtp()
 
 		  case 'n':
 		  case 'N':
-			if (sm_strcasecmp(buf, "noop") == 0)
+			if (SM_STRCASEEQ(buf, "noop"))
 			{
 				printf("250 2.0.0 Ok\r\n");
 				continue;
@@ -673,7 +696,7 @@ dolmtp()
 
 		  case 'q':
 		  case 'Q':
-			if (sm_strcasecmp(buf, "quit") == 0)
+			if (SM_STRCASEEQ(buf, "quit"))
 			{
 				printf("221 2.0.0 Bye\r\n");
 				sm_exit(EX_OK);
@@ -724,7 +747,7 @@ dolmtp()
 				printf("250 2.1.5 Ok\r\n");
 				continue;
 			}
-			else if (sm_strcasecmp(buf, "rset") == 0)
+			else if (SM_STRCASEEQ(buf, "rset"))
 			{
 				printf("250 2.0.0 Ok\r\n");
 
@@ -803,7 +826,7 @@ store(from, inbody)
 #ifdef CONTENTLENGTH
 	HeaderLength = 0;
 	BodyLength = -1;
-#endif /* CONTENTLENGTH */
+#endif
 
 	line[0] = '\0';
 	eline = true;
@@ -959,7 +982,7 @@ store(from, inbody)
 
 	if (fp == NULL || fflush(fp) == EOF || ferror(fp) != 0)
 	{
-		mailerr("451 4.3.0", "Temporary file write error");
+		mailerr("451 4.3.0", "Temporary file flush error");
 		if (fp != NULL)
 			(void) fclose(fp);
 		return -1;
@@ -983,7 +1006,7 @@ deliver(fd, name)
 #ifdef CONTENTLENGTH
 	off_t headerbytes;
 	int readamount;
-#endif /* CONTENTLENGTH */
+#endif
 	char biffmsg[100], buf[8 * 1024];
 	SM_MBDB_T user;
 
@@ -1046,16 +1069,16 @@ deliver(fd, name)
 
 	if (HomeMailFile == NULL)
 	{
-		if (sm_strlcpyn(path, sizeof(path), 
+		if (sm_strlcpyn(path, sizeof(path),
 #if HASHSPOOL
 				4,
-#else /* HASHSPOOL */
+#else
 				3,
-#endif /* HASHSPOOL */
+#endif
 				SpoolPath, "/",
 #if HASHSPOOL
 				hashname(name),
-#endif /* HASHSPOOL */
+#endif
 				name) >= sizeof(path))
 		{
 			exitval = EX_UNAVAILABLE;
@@ -1103,9 +1126,9 @@ deliver(fd, name)
 tryagain:
 #ifdef MAILLOCK
 	p = name;
-#else /* MAILLOCK */
+#else
 	p = path;
-#endif /* MAILLOCK */
+#endif
 	if ((off = lockmbox(p)) != 0)
 	{
 		if (off == EX_TEMPFAIL || e_to_sys(off) == EX_TEMPFAIL)
@@ -1131,7 +1154,7 @@ tryagain:
 		(void) umask(0007);
 		gid = MAILGID;
 		mode |= S_IRGRP|S_IWGRP;
-#endif /* MAILGID */
+#endif
 
 		mbfd = open(path, O_APPEND|O_CREAT|O_EXCL|O_WRONLY,
 			    mode);
@@ -1150,7 +1173,7 @@ tryagain:
 				goto tryagain;
 
 			/* open failed, don't try again */
-			mailerr("450 4.2.0", "%s: %s", path,
+			mailerr("450 4.2.0", "Create %s: %s", path,
 				sm_errstring(save_errno));
 			goto err0;
 		}
@@ -1193,7 +1216,11 @@ tryagain:
 	}
 
 	/* change UID for quota checks */
-	if (setreuid(0, user.mbdb_uid) < 0)
+	if (
+#if MAIL_LOCAL_TEST
+	    (HomeMailFile == NULL || user.mbdb_uid != getuid()) &&
+#endif
+	    setreuid(0, user.mbdb_uid) < 0)
 	{
 		mailerr("450 4.2.0", "setreuid(0, %d): %s (r=%d, e=%d)",
 			(int) user.mbdb_uid, sm_errstring(errno),
@@ -1202,11 +1229,11 @@ tryagain:
 	}
 #ifdef DEBUG
 	fprintf(stderr, "new euid = %d\n", (int) geteuid());
-#endif /* DEBUG */
+#endif
 	mbfd = open(path, O_APPEND|O_WRONLY, 0);
 	if (mbfd < 0)
 	{
-		mailerr("450 4.2.0", "%s: %s", path, sm_errstring(errno));
+		mailerr("450 4.2.0", "Append %s: %s", path, sm_errstring(errno));
 		goto err0;
 	}
 	else if (fstat(mbfd, &fsb) < 0 ||
@@ -1215,9 +1242,9 @@ tryagain:
 		 !S_ISREG(fsb.st_mode) ||
 		 sb.st_dev != fsb.st_dev ||
 		 sb.st_ino != fsb.st_ino ||
-# if HAS_ST_GEN && 0		/* AFS returns random values for st_gen */
+#if HAS_ST_GEN && 0		/* AFS returns random values for st_gen */
 		 sb.st_gen != fsb.st_gen ||
-# endif /* HAS_ST_GEN && 0 */
+#endif
 		 sb.st_uid != fsb.st_uid)
 	{
 		ExitVal = EX_TEMPFAIL;
@@ -1267,7 +1294,7 @@ tryagain:
 	/* Wait until we can get a lock on the file. */
 	if (flock(mbfd, LOCK_EX) < 0)
 	{
-		mailerr("450 4.2.0", "%s: %s", path, sm_errstring(errno));
+		mailerr("450 4.2.0", "Lock %s: %s", path, sm_errstring(errno));
 		goto err1;
 	}
 
@@ -1279,13 +1306,13 @@ tryagain:
 	/* Copy the message into the file. */
 	if (lseek(fd, (off_t) 0, SEEK_SET) == (off_t) -1)
 	{
-		mailerr("450 4.2.0", "Temporary file: %s",
+		mailerr("450 4.2.0", "Temporary file seek error: %s",
 			sm_errstring(errno));
 		goto err1;
 	}
 #ifdef DEBUG
 	fprintf(stderr, "before writing: euid = %d\n", (int) geteuid());
-#endif /* DEBUG */
+#endif
 #ifdef CONTENTLENGTH
 	headerbytes = (BodyLength >= 0) ? HeaderLength : -1 ;
 	for (;;)
@@ -1320,8 +1347,8 @@ tryagain:
 #ifdef EDQUOT
 				if (errno == EDQUOT && BounceQuota)
 					errcode = "552 5.2.2";
-#endif /* EDQUOT */
-				mailerr(errcode, "%s: %s",
+#endif
+				mailerr(errcode, "Write %s: %s",
 					path, sm_errstring(errno));
 				goto err3;
 			}
@@ -1329,7 +1356,7 @@ tryagain:
 	}
 	if (nr < 0)
 	{
-		mailerr("450 4.2.0", "Temporary file: %s",
+		mailerr("450 4.2.0", "Temporary file read error: %s",
 			sm_errstring(errno));
 		goto err3;
 	}
@@ -1337,16 +1364,20 @@ tryagain:
 	/* Flush to disk, don't wait for update. */
 	if (fsync(mbfd) < 0)
 	{
-		mailerr("450 4.2.0", "%s: %s", path, sm_errstring(errno));
+		mailerr("450 4.2.0", "Sync %s: %s", path, sm_errstring(errno));
 err3:
 #ifdef DEBUG
 		fprintf(stderr, "reset euid = %d\n", (int) geteuid());
-#endif /* DEBUG */
+#endif
 		if (mbfd >= 0)
 			(void) ftruncate(mbfd, curoff);
 err1:		if (mbfd >= 0)
 			(void) close(mbfd);
-err0:		(void) setreuid(0, 0);
+err0:
+#if MAIL_LOCAL_TEST
+		if (HomeMailFile == NULL || user.mbdb_uid != getuid())
+#endif
+		(void) setreuid(0, 0);
 		unlockmbox();
 		return;
 	}
@@ -1373,8 +1404,8 @@ err0:		(void) setreuid(0, 0);
 #ifdef EDQUOT
 		if (errno == EDQUOT && BounceQuota)
 			errcode = "552 5.2.2";
-#endif /* EDQUOT */
-		mailerr(errcode, "%s: %s", path, sm_errstring(errno));
+#endif
+		mailerr(errcode, "Close %s: %s", path, sm_errstring(errno));
 		mbfd = open(path, O_WRONLY, 0);
 		if (mbfd < 0 ||
 		    cursize == 0
@@ -1385,9 +1416,9 @@ err0:		(void) setreuid(0, 0);
 		    !S_ISREG(sb.st_mode) ||
 		    sb.st_dev != fsb.st_dev ||
 		    sb.st_ino != fsb.st_ino ||
-# if HAS_ST_GEN && 0		/* AFS returns random values for st_gen */
+#if HAS_ST_GEN && 0		/* AFS returns random values for st_gen */
 		    sb.st_gen != fsb.st_gen ||
-# endif /* HAS_ST_GEN && 0 */
+#endif
 		    sb.st_uid != fsb.st_uid
 		   )
 		{
@@ -1405,7 +1436,11 @@ err0:		(void) setreuid(0, 0);
 	else
 		notifybiff(biffmsg);
 
-	if (setreuid(0, 0) < 0)
+	if (
+#if MAIL_LOCAL_TEST
+	    (HomeMailFile == NULL || user.mbdb_uid != getuid()) &&
+#endif
+	    setreuid(0, 0) < 0)
 	{
 		mailerr("450 4.2.0", "setreuid(0, 0): %s",
 			sm_errstring(errno));
@@ -1413,7 +1448,7 @@ err0:		(void) setreuid(0, 0);
 	}
 #ifdef DEBUG
 	fprintf(stderr, "reset euid = %d\n", (int) geteuid());
-#endif /* DEBUG */
+#endif
 	unlockmbox();
 	if (LMTPMode)
 		printf("250 2.1.5 %s Ok\r\n", name);
@@ -1581,11 +1616,12 @@ void
 usage()
 {
 	ExitVal = EX_USAGE;
-# if _FFR_SPOOL_PATH
+	/* XXX add U to options for USE_EAI */
+#if _FFR_SPOOL_PATH
 	mailerr(NULL, "usage: mail.local [-7] [-b] [-d] [-l] [-f from|-r from] [-h filename] [-p path] user ...");
-# else /* _FFR_SPOOL_PATH */
+#else
 	mailerr(NULL, "usage: mail.local [-7] [-b] [-d] [-l] [-f from|-r from] [-h filename] user ...");
-# endif /* _FFR_SPOOL_PATH */
+#endif
 	sm_exit(ExitVal);
 }
 
@@ -1650,8 +1686,8 @@ hashname(name)
 	MD5_CTX ctx;
 	unsigned char md5[18];
 #  if MAXPATHLEN <= 24
-    ERROR _MAXPATHLEN <= 24
-#  endif /* MAXPATHLEN <= 24 */
+#    ERROR "MAXPATHLEN <= 24"
+#  endif
 	char b64[24];
 	MD5_LONG bits;
 	int j;
@@ -1735,73 +1771,73 @@ e_to_sys(num)
 #endif /* EDQUOT */
 #ifdef EAGAIN
 	  case EAGAIN:		/* Resource temporarily unavailable */
-#endif /* EAGAIN */
+#endif
 #ifdef EBUSY
 	  case EBUSY:		/* Device busy */
-#endif /* EBUSY */
+#endif
 #ifdef EPROCLIM
 	  case EPROCLIM:	/* Too many processes */
-#endif /* EPROCLIM */
+#endif
 #ifdef EUSERS
 	  case EUSERS:		/* Too many users */
-#endif /* EUSERS */
+#endif
 #ifdef ECONNABORTED
 	  case ECONNABORTED:	/* Software caused connection abort */
-#endif /* ECONNABORTED */
+#endif
 #ifdef ECONNREFUSED
 	  case ECONNREFUSED:	/* Connection refused */
-#endif /* ECONNREFUSED */
+#endif
 #ifdef ECONNRESET
 	  case ECONNRESET:	/* Connection reset by peer */
-#endif /* ECONNRESET */
+#endif
 #ifdef EDEADLK
 	  case EDEADLK:		/* Resource deadlock avoided */
-#endif /* EDEADLK */
+#endif
 #ifdef EFBIG
 	  case EFBIG:		/* File too large */
-#endif /* EFBIG */
+#endif
 #ifdef EHOSTDOWN
 	  case EHOSTDOWN:	/* Host is down */
-#endif /* EHOSTDOWN */
+#endif
 #ifdef EHOSTUNREACH
 	  case EHOSTUNREACH:	/* No route to host */
-#endif /* EHOSTUNREACH */
+#endif
 #ifdef EMFILE
 	  case EMFILE:		/* Too many open files */
-#endif /* EMFILE */
+#endif
 #ifdef ENETDOWN
 	  case ENETDOWN:	/* Network is down */
-#endif /* ENETDOWN */
+#endif
 #ifdef ENETRESET
 	  case ENETRESET:	/* Network dropped connection on reset */
-#endif /* ENETRESET */
+#endif
 #ifdef ENETUNREACH
 	  case ENETUNREACH:	/* Network is unreachable */
-#endif /* ENETUNREACH */
+#endif
 #ifdef ENFILE
 	  case ENFILE:		/* Too many open files in system */
-#endif /* ENFILE */
+#endif
 #ifdef ENOBUFS
 	  case ENOBUFS:		/* No buffer space available */
-#endif /* ENOBUFS */
+#endif
 #ifdef ENOMEM
 	  case ENOMEM:		/* Cannot allocate memory */
-#endif /* ENOMEM */
+#endif
 #ifdef ENOSPC
 	  case ENOSPC:		/* No space left on device */
-#endif /* ENOSPC */
+#endif
 #ifdef EROFS
 	  case EROFS:		/* Read-only file system */
-#endif /* EROFS */
+#endif
 #ifdef ESTALE
 	  case ESTALE:		/* Stale NFS file handle */
-#endif /* ESTALE */
+#endif
 #ifdef ETIMEDOUT
 	  case ETIMEDOUT:	/* Connection timed out */
-#endif /* ETIMEDOUT */
+#endif
 #if defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN && EWOULDBLOCK != EDEADLK
 	  case EWOULDBLOCK:	/* Operation would block. */
-#endif /* defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN && EWOULDBLOCK != EDEADLK */
+#endif
 		ExitVal = EX_TEMPFAIL;
 		break;
 
@@ -1848,7 +1884,7 @@ e_to_sys(num)
 
 # if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)mktemp.c	8.1 (Berkeley) 6/4/93";
-# endif /* defined(LIBC_SCCS) && !defined(lint) */
+# endif
 
 # include <sys/types.h>
 # include <sys/stat.h>
