@@ -15,6 +15,8 @@
 
 SM_RCSID("@(#)$Id: alias.c,v 8.221 2013-11-22 20:51:54 ca Exp $")
 
+#include <sm/sendmail.h>
+
 #define SEPARATOR ':'
 # define ALIAS_SPEC_SEPARATORS	" ,/:"
 
@@ -58,7 +60,7 @@ alias(a, sendq, aliaslevel, e)
 	register char *p;
 	char *owner;
 	auto int status = EX_OK;
-	char obuf[MAXNAME + 7];
+	char obuf[MAXNAME_I + 7];
 
 	if (tTd(27, 1))
 		sm_dprintf("alias(%s)\n", a->q_user);
@@ -186,11 +188,12 @@ alias(a, sendq, aliaslevel, e)
 	e->e_flags |= EF_SENDRECEIPT;
 	a->q_flags |= QDELIVERED|QEXPANDED;
 }
+
 /*
 **  ALIASLOOKUP -- look up a name in the alias file.
 **
 **	Parameters:
-**		name -- the name to look up.
+**		name -- the name to look up [i]
 **		pstat -- a pointer to a place to put the status.
 **		av -- argument for %1 expansion.
 **
@@ -198,9 +201,6 @@ alias(a, sendq, aliaslevel, e)
 **		the value of name.
 **		NULL if unknown.
 **
-**	Side Effects:
-**		none.
-**
 **	Warnings:
 **		The return value will be trashed across calls.
 */
@@ -212,10 +212,16 @@ aliaslookup(name, pstat, av)
 	char *av;
 {
 	static MAP *map = NULL;
+	char *res;
 #if _FFR_ALIAS_DETAIL
 	int i;
 	char *argv[4];
-#endif /* _FFR_ALIAS_DETAIL */
+#else
+# define argv NULL
+#endif
+#if _FFR_8BITENVADDR
+	char buf[MAXNAME];	/* EAI:ok */
+#endif
 
 	if (map == NULL)
 	{
@@ -228,8 +234,13 @@ aliaslookup(name, pstat, av)
 	DYNOPENMAP(map);
 
 	/* special case POstMastER -- always use lower case */
-	if (sm_strcasecmp(name, "postmaster") == 0)
+	if (SM_STRCASEEQ(name, "postmaster"))
 		name = "postmaster";
+#if _FFR_8BITENVADDR
+	(void) dequote_internal_chars(name, buf, sizeof(buf));
+	/* check length? */
+	name = buf;
+#endif /* _FFR_8BITENVADDR */
 
 #if _FFR_ALIAS_DETAIL
 	i = 0;
@@ -240,11 +251,14 @@ aliaslookup(name, pstat, av)
 	if (av != NULL && *av == '+')
 		argv[i++] = av + 1;
 	argv[i++] = NULL;
-	return (*map->map_class->map_lookup)(map, name, argv, pstat);
-#else /* _FFR_ALIAS_DETAIL */
-	return (*map->map_class->map_lookup)(map, name, NULL, pstat);
 #endif /* _FFR_ALIAS_DETAIL */
+	res = (*map->map_class->map_lookup)(map, name, argv, pstat);
+#if _FFR_8BITENVADDR
+	/* map_lookup() does a map_rewrite(), so no quoting here */
+#endif
+	return res;
 }
+
 /*
 **  SETALIAS -- set up an alias map
 **
@@ -273,7 +287,7 @@ setalias(spec)
 	{
 		char buf[50];
 
-		while (isascii(*p) && isspace(*p))
+		while (SM_ISSPACE(*p))
 			p++;
 		if (*p == '\0')
 			break;
@@ -507,7 +521,7 @@ rebuildaliases(map, automatic)
 	sigfunc_t oldsigint, oldsigquit;
 #ifdef SIGTSTP
 	sigfunc_t oldsigtstp;
-#endif /* SIGTSTP */
+#endif
 
 	if (!bitset(MCF_REBUILDABLE, map->map_class->map_cflags))
 		return false;
@@ -569,7 +583,7 @@ rebuildaliases(map, automatic)
 	oldsigquit = sm_signal(SIGQUIT, SIG_IGN);
 #ifdef SIGTSTP
 	oldsigtstp = sm_signal(SIGTSTP, SIG_IGN);
-#endif /* SIGTSTP */
+#endif
 
 	if (map->map_class->map_open(map, O_RDWR))
 	{
@@ -601,6 +615,16 @@ rebuildaliases(map, automatic)
 	/* add distinguished entries and close the database */
 	if (bitset(MF_OPEN, map->map_mflags))
 	{
+#if _FFR_TESTS
+		if (tTd(78, 101))
+		{
+			int sl;
+
+			sl = tTdlevel(78) - 100;
+			sm_dprintf("rebuildaliases: sleep=%d\n", sl);
+			sleep(sl);
+		}
+#endif
 		map->map_mflags |= MF_CLOSING;
 		map->map_class->map_close(map);
 		map->map_mflags &= ~(MF_OPEN|MF_WRITABLE|MF_CLOSING);
@@ -611,9 +635,58 @@ rebuildaliases(map, automatic)
 	(void) sm_signal(SIGQUIT, oldsigquit);
 #ifdef SIGTSTP
 	(void) sm_signal(SIGTSTP, oldsigtstp);
-#endif /* SIGTSTP */
+#endif
 	return success;
 }
+
+
+/*
+**  CONTLINE -- handle potential continuation line
+**
+**	Parameters:
+**		fp -- file to read
+**		line -- current line
+**
+**	Returns:
+**		pointer to end of current line if there is a continuation line
+**		NULL otherwise
+**
+**	Side Effects:
+**		Modifies line if it is a continuation line
+*/
+
+static char *contline __P((SM_FILE_T *, char *));
+static char *
+contline(fp, line)
+	SM_FILE_T *fp;
+	char *line;
+{
+	char *p;
+	int c;
+
+	if ((p = strchr(line, '\n')) != NULL && p > line && p[-1] == '\\')
+	{
+		*p = '\0';
+		*--p = '\0';
+		return p;
+	}
+
+	c = sm_io_getc(fp, SM_TIME_DEFAULT);
+	if (!sm_io_eof(fp))
+		(void) sm_io_ungetc(fp, SM_TIME_DEFAULT, c);
+	if (c == ' ' || c == '\t')
+	{
+		char *nlp;
+
+		p = line;
+		nlp = &p[strlen(p)];
+		if (nlp > p && nlp[-1] == '\n')
+			*--nlp = '\0';
+		return nlp;
+	}
+	return NULL;
+}
+
 /*
 **  READALIASES -- read and process the alias file.
 **
@@ -647,7 +720,13 @@ readaliases(map, af, announcestats, logstats)
 	bool skipping;
 	long naliases, bytes, longest;
 	ADDRESS al, bl;
-	char line[BUFSIZ];
+	char lbuf[BUFSIZ];
+	char *line;
+#if _FFR_8BITENVADDR
+	char lhsbuf[MAXNAME];	/* EAI:ok */
+	char rhsbuf[BUFSIZ];
+	int len;
+#endif
 
 	/*
 	**  Read and interpret lines
@@ -657,40 +736,76 @@ readaliases(map, af, announcestats, logstats)
 	LineNumber = 0;
 	naliases = bytes = longest = 0;
 	skipping = false;
-	while (sm_io_fgets(af, SM_TIME_DEFAULT, line, sizeof(line)) >= 0)
+	line = NULL;
+
+	while (sm_io_fgets(af, SM_TIME_DEFAULT, lbuf, sizeof(lbuf)) >= 0)
 	{
 		int lhssize, rhssize;
 		int c;
+		char *newp;
 
 		LineNumber++;
-		p = strchr(line, '\n');
 
 		/* XXX what if line="a\\" ? */
-		while (p != NULL && p > line && p[-1] == '\\')
+		line = lbuf;
+		p = line;
+		while ((newp = contline(af, line)) != NULL)
 		{
-			p--;
-			if (sm_io_fgets(af, SM_TIME_DEFAULT, p,
-					SPACELEFT(line, p)) < 0)
+			p = newp;
+			if ((c = sm_io_fgets(af, SM_TIME_DEFAULT, p,
+					SPACELEFT(lbuf, p))) < 0)
+			{
 				break;
+			}
 			LineNumber++;
-			p = strchr(p, '\n');
 		}
+#if _FFR_8BITENVADDR
+		if (SMTPUTF8 || EightBitAddrOK)
+		{
+			if (line != lbuf)
+				SM_FREE(line);
+			len = 0;
+			line = quote_internal_chars(lbuf, NULL, &len, NULL);
+		}
+		else
+#endif
+		/* "else" in #if code above */
+		line = lbuf;
+
+		p = strchr(line, '\n');
 		if (p != NULL)
 			*p = '\0';
 		else if (!sm_io_eof(af))
 		{
+			int prev;
+			bool cl;
+
 			errno = 0;
 			syserr("554 5.3.0 alias line too long");
 
-			/* flush to end of line */
-			while ((c = sm_io_getc(af, SM_TIME_DEFAULT)) !=
-				SM_IO_EOF && c != '\n')
-				continue;
+			prev = '\0';
+			cl = false;
+
+			do {
+				/* flush to end of "virtual" line */
+				while ((c = sm_io_getc(af, SM_TIME_DEFAULT)) !=
+					SM_IO_EOF && c != '\n')
+				{
+					prev = c;
+				}
+				cl = ('\\' == prev && '\n' == c);
+				if (!cl)
+				{
+					c = sm_io_getc(af, SM_TIME_DEFAULT);
+					if (!sm_io_eof(af))
+						(void) sm_io_ungetc(af, SM_TIME_DEFAULT, c);
+					cl = (c == ' ' || c == '\t');
+				}
+			} while (cl);
 
-			/* skip any continuation lines */
-			skipping = true;
 			continue;
 		}
+
 		switch (line[0])
 		{
 		  case '#':
@@ -723,6 +838,7 @@ readaliases(map, af, announcestats, logstats)
 			syserr("554 5.3.5 missing colon");
 			continue;
 		}
+/* XXX line must be [i] */
 		if (parseaddr(line, &al, RF_COPYALL, ':', NULL, CurEnv, true)
 		    == NULL)
 		{
@@ -736,10 +852,9 @@ readaliases(map, af, announcestats, logstats)
 		**	'p' points to the text of the RHS.
 		*/
 
-		while (isascii(*p) && isspace(*p))
+		while (SM_ISSPACE(*p))
 			p++;
 		rhs = p;
-		for (;;)
 		{
 			register char *nlp;
 
@@ -754,11 +869,11 @@ readaliases(map, af, announcestats, logstats)
 				{
 					auto char *delimptr;
 
-					while ((isascii(*p) && isspace(*p)) ||
-								*p == ',')
+					while ((SM_ISSPACE(*p)) || *p == ',')
 						p++;
 					if (*p == '\0')
 						break;
+/* XXX p must be [i] */
 					if (parseaddr(p, &bl, RF_COPYNONE, ',',
 						      &delimptr, CurEnv, true)
 					    == NULL)
@@ -770,31 +885,7 @@ readaliases(map, af, announcestats, logstats)
 			{
 				p = nlp;
 			}
-
-			/* see if there should be a continuation line */
-			c = sm_io_getc(af, SM_TIME_DEFAULT);
-			if (!sm_io_eof(af))
-				(void) sm_io_ungetc(af, SM_TIME_DEFAULT, c);
-			if (c != ' ' && c != '\t')
-				break;
-
-			/* read continuation line */
-			if (sm_io_fgets(af, SM_TIME_DEFAULT, p,
-					sizeof(line) - (p-line)) < 0)
-				break;
-			LineNumber++;
-
-			/* check for line overflow */
-			if (strchr(p, '\n') == NULL && !sm_io_eof(af))
-			{
-				usrerr("554 5.3.5 alias too long");
-				while ((c = sm_io_getc(af, SM_TIME_DEFAULT))
-				       != SM_IO_EOF && c != '\n')
-					continue;
-				skipping = true;
-				break;
-			}
-		}
+		} while (0);
 
 		if (skipping)
 			continue;
@@ -812,8 +903,8 @@ readaliases(map, af, announcestats, logstats)
 		**	Special case pOStmaStER -- always make it lower case.
 		*/
 
-		if (sm_strcasecmp(al.q_user, "postmaster") == 0)
-			makelower(al.q_user);
+		if (SM_STRCASEEQ(al.q_user, "postmaster"))
+			makelower_a(&al.q_user, CurEnv->e_rpool);
 
 		lhssize = strlen(al.q_user);
 		rhssize = strlen(rhs);
@@ -821,17 +912,26 @@ readaliases(map, af, announcestats, logstats)
 		{
 			/* is RHS empty (just spaces)? */
 			p = rhs;
-			while (isascii(*p) && isspace(*p))
+			while (SM_ISSPACE(*p))
 				p++;
 		}
 		if (rhssize == 0 || *p == '\0')
 		{
 			syserr("554 5.3.5 %.40s... missing value for alias",
 			       line);
-
 		}
 		else
 		{
+#if _FFR_8BITENVADDR
+			if (SMTPUTF8 || EightBitAddrOK)
+			{
+				dequote_internal_chars(al.q_user, lhsbuf, sizeof(lhsbuf));
+				dequote_internal_chars(rhs, rhsbuf, sizeof(rhsbuf));
+				map->map_class->map_store(map, lhsbuf, rhsbuf);
+			}
+			else
+#endif
+			/* "else" in #if code above */
 			map->map_class->map_store(map, al.q_user, rhs);
 
 			/* statistics */
@@ -840,19 +940,6 @@ readaliases(map, af, announcestats, logstats)
 			if (rhssize > longest)
 				longest = rhssize;
 		}
-
-#if 0
-	/*
-	**  address strings are now stored in the envelope rpool,
-	**  and therefore cannot be freed.
-	*/
-		if (al.q_paddr != NULL)
-			sm_free(al.q_paddr);  /* disabled */
-		if (al.q_host != NULL)
-			sm_free(al.q_host);  /* disabled */
-		if (al.q_user != NULL)
-			sm_free(al.q_user);  /* disabled */
-#endif /* 0 */
 	}
 
 	CurEnv->e_to = NULL;
@@ -873,8 +960,7 @@ readaliases(map, af, announcestats, logstats)
 **	Parameters:
 **		user -- the name of the user who's mail we would like
 **			to forward to.  It must have been verified --
-**			i.e., the q_home field must have been filled
-**			in.
+**			i.e., the q_home field must have been filled in.
 **		sendq -- a pointer to the head of the send queue to
 **			put this user's aliases in.
 **		aliaslevel -- the current alias nesting depth.
