@@ -16,8 +16,9 @@
 /*
 **  CONF.H -- All user-configurable parameters for sendmail
 **
-**	Send updates to sendmail@Sendmail.ORG so they will be
-**	included in the next release.
+**	Send updates to sendmail-YYYY@support.sendmail.org
+**	(replace YYYY with the current year)
+**	so they will be included in the next release.
 */
 
 #ifndef CONF_H
@@ -33,7 +34,7 @@ struct rusage;	/* forward declaration to get gcc to shut up in wait.h */
 # ifndef __QNX__
 /* in QNX this grabs bogus LOCK_* manifests */
 #  include <sys/file.h>
-# endif /* ! __QNX__ */
+# endif
 # include <sys/wait.h>
 # include <limits.h>
 # include <fcntl.h>
@@ -108,7 +109,7 @@ struct rusage;	/* forward declaration to get gcc to shut up in wait.h */
 #define MAXSHORTSTR	203		/* max short string length */
 #define MAXMACNAMELEN	25		/* max macro name length */
 #define MAXMACROID	0377		/* max macro id number */
-					/* Must match (BITMAPBITS - 1) */
+	/* Must match (BITMAPBITS - 1), checked in macro.c */
 #ifndef MAXHDRSLEN
 # define MAXHDRSLEN	(32 * 1024)	/* max size of message headers */
 #endif
@@ -131,7 +132,7 @@ struct rusage;	/* forward declaration to get gcc to shut up in wait.h */
 	/* must be less than BITMAPBITS for DoQueueRun */
 #endif
 #if MAXQUEUEGROUPS >= BITMAPBITS
-  ERROR _MAXQUEUEGROUPS must be less than _BITMAPBITS
+# ERROR "MAXQUEUEGROUPS must be less than BITMAPBITS"
 #endif
 
 #ifndef MAXWORKGROUPS
@@ -152,9 +153,9 @@ struct rusage;	/* forward declaration to get gcc to shut up in wait.h */
 # ifndef AUTH_MECHANISMS
 #  if STARTTLS
 #   define AUTH_MECHANISMS	"EXTERNAL GSSAPI KERBEROS_V4 DIGEST-MD5 CRAM-MD5"
-#  else /* STARTTLS */
+#  else
 #   define AUTH_MECHANISMS	"GSSAPI KERBEROS_V4 DIGEST-MD5 CRAM-MD5"
-#  endif /* STARTTLS */
+#  endif
 # endif /* ! AUTH_MECHANISMS */
 #endif /* SASL */
 
@@ -239,6 +240,14 @@ struct rusage;	/* forward declaration to get gcc to shut up in wait.h */
 **  End of site-specific configuration.
 **********************************************************************/
 
+#if CDB >= 2
+# define CDBEXT ".db"
+# define CDBext "db"
+#else
+# define CDBEXT ".cdb"
+# define CDBext "cdb"
+#endif
+
 #include <sm/conf.h>
 
 #endif /* ! CONF_H */
