@@ -18,7 +18,7 @@ SM_RCSID("@(#)$Id: macro.c,v 8.108 2013-11-22 20:51:55 ca Exp $")
 #include <sm/sendmail.h>
 #if MAXMACROID != (BITMAPBITS - 1)
 	ERROR Read the comment in conf.h
-#endif /* MAXMACROID != (BITMAPBITS - 1) */
+#endif
 
 static char	*MacroName[MAXMACROID + 1];	/* macro id to name table */
 
@@ -159,23 +159,17 @@ initmacros(e)
 
 /*
 **  EXPAND/DOEXPAND -- macro expand a string using $x escapes.
-**
-**	After expansion, the expansion will be in external form (that is,
-**	there will be no sendmail metacharacters and METAQUOTEs will have
-**	been stripped out).
+**	(including conditionals, e.g., $?x Y $| N $.)
 **
 **	Parameters:
-**		s -- the string to expand.
-**		buf -- the place to put the expansion.
+**		s -- the string to expand. [i]
+**		buf -- the place to put the expansion. [i]
 **		bufsize -- the size of the buffer.
 **		explevel -- the depth of expansion (doexpand only)
 **		e -- envelope in which to work.
 **
 **	Returns:
 **		none.
-**
-**	Side Effects:
-**		none.
 */
 
 static void doexpand __P(( char *, char *, size_t, int, ENVELOPE *));
@@ -383,7 +377,7 @@ mactabclear(mac)
 	if (mac->mac_rpool == NULL)
 	{
 		for (i = 0; i < MAXMACROID; i++)
-			SM_FREE_CLR(mac->mac_table[i]);
+			SM_FREE(mac->mac_table[i]);
 	}
 	memset((char *) mac, '\0', sizeof(*mac));
 }
@@ -422,9 +416,9 @@ mactabclear(mac)
 void
 #if SM_HEAP_CHECK
 macdefine_tagged(mac, vclass, id, value, file, line, grp)
-#else /* SM_HEAP_CHECK */
+#else
 macdefine(mac, vclass, id, value)
-#endif /* SM_HEAP_CHECK */
+#endif
 	MACROS_T *mac;
 	ARGCLASS_T vclass;
 	int id;
@@ -433,7 +427,7 @@ macdefine(mac, vclass, id, value)
 	char *file;
 	int line;
 	int grp;
-#endif /* SM_HEAP_CHECK */
+#endif
 {
 	char *newvalue;
 
@@ -447,6 +441,10 @@ macdefine(mac, vclass, id, value)
 		xputs(sm_debug_file(), value);
 		sm_dprintf(")\n");
 	}
+#if USE_EAI && 0
+	if (('j' == id || 'm' == id) && !addr_is_ascii(value))
+		return an error/warning to caller and let them handle it.
+#endif
 
 	if (mac->mac_rpool == NULL)
 	{
@@ -466,9 +464,9 @@ macdefine(mac, vclass, id, value)
 		{
 #if SM_HEAP_CHECK
 			newvalue = sm_strdup_tagged_x(value, file, line, 0);
-#else /* SM_HEAP_CHECK */
+#else
 			newvalue = sm_strdup_x(value);
-#endif /* SM_HEAP_CHECK */
+#endif
 			setbitn(id, mac->mac_allocated);
 		}
 		mac->mac_table[id] = newvalue;
@@ -566,6 +564,15 @@ macvalue(n, e)
 			break;
 		e = e->e_parent;
 	}
+#if _FFR_BLANKENV_MACV
+	if (LOOKUP_MACRO_IN_BLANKENV && e != &BlankEnvelope)
+	{
+		char *p = BlankEnvelope.e_macro.mac_table[n];
+
+		if (p != NULL)
+			return p;
+	}
+#endif
 	return GlobalMacros.mac_table[n];
 }
 
