@@ -155,6 +155,7 @@ parseaddr(addr, a, flags, delim, delimptr, e, isrcpt)
 
 	a = buildaddr(pvp, a, flags, e);
 #if _FFR_8BITENVADDR
+	if (NULL != a->q_user)
 	{
 		int len = 0;
 
@@ -267,7 +268,7 @@ parseaddr(addr, a, flags, delim, delimptr, e, isrcpt)
 **		isrcpt -- true iff the address is for a recipient.
 **
 **	Returns:
-**		true -- if the address has characters that are reservered
+**		true -- if the address has characters that are reserved
 **			for macros or is too long.
 **		false -- otherwise.
 */
