@@ -15,6 +15,8 @@
 
 SM_RCSID("@(#)$Id: savemail.c,v 8.319 2013-11-22 20:51:56 ca Exp $")
 
+#include <sm/sendmail.h>
+
 static bool	errbody __P((MCI *, ENVELOPE *, char *));
 static bool	pruneroute __P((char *));
 
@@ -180,9 +182,9 @@ savemail(e, sendbody)
 
 #if USE_TTYPATH
 			p = ttypath();
-#else /* USE_TTYPATH */
+#else
 			p = NULL;
-#endif /* USE_TTYPATH */
+#endif
 
 			if (p == NULL || sm_io_reopen(SmFtStdio,
 						      SM_TIME_DEFAULT,
@@ -513,7 +515,7 @@ returntosender(msg, returnq, flags, e)
 	static int returndepth = 0;
 	register ADDRESS *q;
 	char *p;
-	char buf[MAXNAME + 1];
+	char buf[MAXNAME + 1];	/* EAI:ok */
 
 	if (returnq == NULL)
 		return -1;
@@ -524,7 +526,7 @@ returntosender(msg, returnq, flags, e)
 	if (tTd(6, 1))
 	{
 		sm_dprintf("\n*** Return To Sender: msg=\"%s\", depth=%d, e=%p, returnq=",
-			msg, returndepth, e);
+			msg, returndepth, (void *)e);
 		printaddr(sm_debug_file(), returnq, true);
 		if (tTd(6, 20))
 		{
@@ -584,7 +586,7 @@ returntosender(msg, returnq, flags, e)
 #if _FFR_BOUNCE_QUEUE
 	if (BounceQueue != NOQGRP)
 		ee->e_qgrp = ee->e_dfqgrp = BounceQueue;
-#endif /* _FFR_BOUNCE_QUEUE */
+#endif
 	if (!setnewqueue(ee))
 	{
 		syserr("554 5.3.0 returntosender: cannot select queue for %s",
@@ -598,7 +600,7 @@ returntosender(msg, returnq, flags, e)
 #if NAMED_BIND
 	_res.retry = TimeOuts.res_retry[RES_TO_FIRST];
 	_res.retrans = TimeOuts.res_retrans[RES_TO_FIRST];
-#endif /* NAMED_BIND */
+#endif
 	for (q = returnq; q != NULL; q = q->q_next)
 	{
 		if (QS_IS_BADADDR(q->q_state))
@@ -637,9 +639,9 @@ returntosender(msg, returnq, flags, e)
 		(void) sm_snprintf(buf, sizeof(buf),
 #if DSN
 				"multipart/report; report-type=delivery-status;\n\tboundary=\"%s\"",
-#else /* DSN */
+#else
 				"multipart/mixed; boundary=\"%s\"",
-#endif /* DSN */
+#endif
 				ee->e_msgboundary);
 		addheader("Content-Type", buf, 0, ee, true);
 
@@ -685,6 +687,7 @@ returntosender(msg, returnq, flags, e)
 
 	/* fake up an address header for the from person */
 	expand("\201n", buf, sizeof(buf), e);
+/* XXX buf must be [i] */
 	if (parseaddr(buf, &ee->e_from,
 		      RF_COPYALL|RF_SENDERADDR, '\0', NULL, e, false) == NULL)
 	{
@@ -697,6 +700,12 @@ returntosender(msg, returnq, flags, e)
 	ee->e_from.q_flags |= QPINGONFAILURE;
 	ee->e_sender = ee->e_from.q_paddr;
 
+#if USE_EAI
+	/* always? when is this really needed? */
+	if (e->e_smtputf8)
+		ee->e_smtputf8 = true;
+#endif
+
 	/* push state into submessage */
 	CurEnv = ee;
 	macdefine(&ee->e_macro, A_PERM, 'f', "\201n");
@@ -744,6 +753,44 @@ returntosender(msg, returnq, flags, e)
 	return ret;
 }
 
+
+/*
+**  DSNTYPENAME -- Returns the DSN name of the addrtype for this address
+**
+**	Sendmail's addrtypes are largely in different universes, and
+**	'fred' may be a valid address in different addrtype
+**	universes.
+**
+**	EAI extends the rfc822 universe rather than introduce a new
+**	universe.  Because of that, sendmail uses the rfc822 addrtype,
+**	but names it utf-8 when the EAI DSN extension requires that.
+**
+**	Parameters:
+**		addrtype -- address type
+**		addr -- the address
+**
+**	Returns:
+**		type for DSN
+**
+*/
+
+static const char *dsntypename __P((const char *, const char *));
+
+static const char *
+dsntypename(addrtype, addr)
+	const char *addrtype;
+	const char *addr;
+{
+	if (sm_strcasecmp(addrtype, "rfc822") != 0)
+		return addrtype;
+#if USE_EAI
+	if (!addr_is_ascii(addr))
+		return "utf-8";
+#endif
+	return "rfc822";
+}
+
+
 /*
 **  ERRBODY -- output the body of an error message.
 **
@@ -1082,7 +1129,13 @@ errbody(mci, e, separator)
 		(void) sm_strlcpyn(buf, sizeof(buf), 2, "--", e->e_msgboundary);
 		if (!putline("", mci) ||
 		    !putline(buf, mci) ||
-		    !putline("Content-Type: message/delivery-status", mci) ||
+		    !putline(
+# if USE_EAI
+			e->e_parent->e_smtputf8
+			? "Content-Type: message/global-delivery-status"
+			:
+# endif
+			"Content-Type: message/delivery-status", mci) ||
 		    !putline("", mci))
 			goto writeerr;
 
@@ -1217,13 +1270,14 @@ errbody(mci, e, separator)
 				else
 					p = "rfc822";
 
-				if (sm_strcasecmp(p, "rfc822") == 0 &&
+				if (SM_STRCASEEQ(p, "rfc822") &&
 				    strchr(q->q_user, '@') == NULL)
 				{
 					(void) sm_snprintf(actual,
 							   sizeof(actual),
 							   "%s; %.700s@%.100s",
-							   p, q->q_user,
+							   dsntypename(p, q->q_user),
+							   q->q_user,
 							   MyHostName);
 				}
 				else
@@ -1231,7 +1285,8 @@ errbody(mci, e, separator)
 					(void) sm_snprintf(actual,
 							   sizeof(actual),
 							   "%s; %.800s",
-							   p, q->q_user);
+							   dsntypename(p, q->q_user),
+							   q->q_user);
 				}
 			}
 
@@ -1248,6 +1303,23 @@ errbody(mci, e, separator)
 									   actual);
 			}
 
+# if USE_EAI
+			if (sm_strncasecmp("rfc822;", q->q_finalrcpt, 7) == 0 &&
+			    !addr_is_ascii(q->q_user))
+			{
+				char *a;
+				char utf8rcpt[1024];
+
+				a = strchr(q->q_finalrcpt, ';');
+				while(*a == ';' || *a == ' ')
+					a++;
+				sm_snprintf(utf8rcpt, sizeof(utf8rcpt),
+					    "utf-8; %.800s", a);
+				q->q_finalrcpt = sm_rpool_strdup_x(e->e_rpool,
+								   utf8rcpt);
+			}
+# endif /* USE_EAI */
+
 			if (q->q_finalrcpt != NULL)
 			{
 				(void) sm_snprintf(buf, sizeof(buf),
@@ -1373,9 +1445,20 @@ errbody(mci, e, separator)
 
 			if (!putline(buf, mci))
 				goto writeerr;
-			(void) sm_strlcpyn(buf, sizeof(buf), 2, "Content-Type: ",
-					sendbody ? "message/rfc822"
-						 : "text/rfc822-headers");
+#if USE_EAI
+			if (e->e_parent->e_smtputf8)
+				(void) sm_strlcpyn(buf, sizeof(buf), 2,
+						   "Content-Type: message/global",
+						   sendbody ? "" : "-headers");
+			else
+#endif /* USE_EAI */
+			/* "else" in #if code above */
+			{
+				(void) sm_strlcpyn(buf, sizeof(buf), 2,
+						"Content-Type: ",
+						sendbody ? "message/rfc822"
+							 : "text/rfc822-headers");
+			}
 			if (!putline(buf, mci))
 				goto writeerr;
 
@@ -1725,7 +1808,7 @@ isatom(s)
 {
 	int c;
 
-	if (s == NULL || *s == '\0')
+	if (SM_IS_EMPTY(s))
 		return false;
 	while ((c = *s++) != '\0')
 	{
@@ -1804,8 +1887,8 @@ pruneroute(addr)
 
 	while (start != NULL)
 	{
-		if (getmxrr(hostbuf, mxhosts, NULL, TRYFALLBACK, &rcode, NULL)
-		    > 0)
+		if (getmxrr(hostbuf, mxhosts, NULL, TRYFALLBACK, &rcode, NULL,
+			    -1) > 0)
 		{
 			(void) sm_strlcpy(addr + 1, start + 1,
 					  strlen(addr) - 1);
