@@ -19,7 +19,7 @@ SM_RCSID("@(#)$Id: sfsasl.c,v 8.121 2013-11-22 20:51:56 ca Exp $")
 /* allow to disable error handling code just in case... */
 #ifndef DEAL_WITH_ERROR_SSL
 # define DEAL_WITH_ERROR_SSL	1
-#endif /* ! DEAL_WITH_ERROR_SSL */
+#endif
 
 #if SASL
 # include "sfsasl.h"
@@ -154,11 +154,7 @@ sasl_close(fp)
 	so = (struct sasl_obj *) fp->f_cookie;
 	if (so == NULL)
 		return 0;
-	if (so->fp != NULL)
-	{
-		sm_io_close(so->fp, SM_TIME_DEFAULT);
-		so->fp = NULL;
-	}
+	SM_CLOSE_FP(so->fp);
 	sm_free(so);
 	so = NULL;
 	return 0;
@@ -193,9 +189,9 @@ sasl_read(fp, buf, size)
 	ssize_t len;
 # if SASL >= 20000
 	static const char *outbuf = NULL;
-# else /* SASL >= 20000 */
+# else
 	static char *outbuf = NULL;
-# endif /* SASL >= 20000 */
+# endif
 	static unsigned int outlen = 0;
 	static unsigned int offset = 0;
 	struct sasl_obj *so = (struct sasl_obj *) fp->f_cookie;
@@ -214,9 +210,9 @@ sasl_read(fp, buf, size)
 
 # if SASL >= 20000
 	while (outlen == 0)
-# else /* SASL >= 20000 */
+# else
 	while (outbuf == NULL && outlen == 0)
-# endif /* SASL >= 20000 */
+# endif
 	{
 		len = sm_io_read(so->fp, SM_TIME_DEFAULT, buf, size);
 		if (len <= 0)
@@ -255,7 +251,7 @@ sasl_read(fp, buf, size)
 		(void) memcpy(buf, outbuf + offset, (size_t) len);
 # if SASL < 20000
 		SASL_DEALLOC(outbuf);
-# endif /* SASL < 20000 */
+# endif
 		outbuf = NULL;
 		offset = 0;
 		outlen = 0;
@@ -287,9 +283,9 @@ sasl_write(fp, buf, size)
 	int result;
 # if SASL >= 20000
 	const char *outbuf;
-# else /* SASL >= 20000 */
+# else
 	char *outbuf;
-# endif /* SASL >= 20000 */
+# endif
 	unsigned int outlen, *maxencode;
 	size_t ret = 0, total = 0;
 	struct sasl_obj *so = (struct sasl_obj *) fp->f_cookie;
@@ -339,7 +335,7 @@ sasl_write(fp, buf, size)
 		}
 # if SASL < 20000
 		SASL_DEALLOC(outbuf);
-# endif /* SASL < 20000 */
+# endif
 	}
 	return size;
 }
@@ -544,11 +540,7 @@ tls_close(fp)
 	so = (struct tls_obj *) fp->f_cookie;
 	if (so == NULL)
 		return 0;
-	if (so->fp != NULL)
-	{
-		sm_io_close(so->fp, SM_TIME_DEFAULT);
-		so->fp = NULL;
-	}
+	SM_CLOSE_FP(so->fp);
 	sm_free(so);
 	so = NULL;
 	return 0;
@@ -672,11 +664,11 @@ tls_retry(ssl, rfd, wfd, tlsstart, timeout, err, where)
 }
 
 /* errno to force refill() etc to stop (see IS_IO_ERROR()) */
-#ifdef ETIMEDOUT
-# define SM_ERR_TIMEOUT	ETIMEDOUT
-#else /* ETIMEDOUT */
-# define SM_ERR_TIMEOUT	EIO
-#endif /* ETIMEDOUT */
+# ifdef ETIMEDOUT
+#  define SM_ERR_TIMEOUT	ETIMEDOUT
+# else
+#  define SM_ERR_TIMEOUT	EIO
+# endif
 
 /*
 **  SET_TLS_RD_TMO -- read secured information for the caller
@@ -768,15 +760,12 @@ tls_read(fp, buf, size)
 		if (r == 0 && errno == 0) /* out of protocol EOF found */
 			break;
 		err = "syscall error";
-/*
-		get_last_socket_error());
-*/
 		break;
 	  case SSL_ERROR_SSL:
-#if DEAL_WITH_ERROR_SSL
+# if DEAL_WITH_ERROR_SSL
 		if (r == 0 && errno == 0) /* out of protocol EOF found */
 			break;
-#endif /* DEAL_WITH_ERROR_SSL */
+# endif
 		err = "generic SSL error";
 
 		if (LogLevel > 9)
@@ -790,11 +779,11 @@ tls_read(fp, buf, size)
 			tlslogerr(pri, 9, "read");
 		}
 
-#if DEAL_WITH_ERROR_SSL
+# if DEAL_WITH_ERROR_SSL
 		/* avoid repeated calls? */
 		if (r == 0)
 			r = -1;
-#endif /* DEAL_WITH_ERROR_SSL */
+# endif
 		break;
 	}
 	if (err != NULL)
@@ -893,9 +882,6 @@ tls_write(fp, buf, size)
 		if (r == 0 && errno == 0) /* out of protocol EOF found */
 			break;
 		err = "syscall error";
-/*
-		get_last_socket_error());
-*/
 		break;
 	  case SSL_ERROR_SSL:
 		err = "generic SSL error";
@@ -904,11 +890,11 @@ tls_write(fp, buf, size)
 */
 		tlslogerr(LOG_WARNING, 9, "write");
 
-#if DEAL_WITH_ERROR_SSL
+# if DEAL_WITH_ERROR_SSL
 		/* avoid repeated calls? */
 		if (r == 0)
 			r = -1;
-#endif /* DEAL_WITH_ERROR_SSL */
+# endif
 		break;
 	}
 	if (err != NULL)
