@@ -13,7 +13,6 @@
 SM_RCSID("@(#)$Id: tls.c,v 8.127 2013-11-27 02:51:11 gshapiro Exp $")
 
 #if STARTTLS
-# include <tls.h>
 # include <openssl/err.h>
 # include <openssl/bio.h>
 # include <openssl/pem.h>
@@ -24,23 +23,35 @@ SM_RCSID("@(#)$Id: tls.c,v 8.127 2013-11-27 02:51:11 gshapiro Exp $")
 # if _FFR_TLS_ALTNAMES
 #  include <openssl/x509v3.h>
 # endif
+# include <tls.h>
 
 # if defined(OPENSSL_VERSION_NUMBER) && OPENSSL_VERSION_NUMBER <= 0x00907000L
-# ERROR: OpenSSL version OPENSSL_VERSION_NUMBER is unsupported.
+#  ERROR "OpenSSL version OPENSSL_VERSION_NUMBER is unsupported."
 # endif
 
-# if OPENSSL_VERSION_NUMBER >= 0x10100000L && OPENSSL_VERSION_NUMBER < 0x20000000L
+/*
+**  *SSL version numbers:
+**  OpenSSL 0.9 - 1.1 (so far), 3.0
+**  LibreSSL 2.0 (0x20000000L - part of "These will never change")
+*/
+
+# if (OPENSSL_VERSION_NUMBER >= 0x10100000L && OPENSSL_VERSION_NUMBER < 0x20000000L) || OPENSSL_VERSION_NUMBER >= 0x30000000L || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER >= 0x2070000fL)
 #  define MTA_HAVE_DH_set0_pqg 1
 #  define MTA_HAVE_DSA_GENERATE_EX	1
+#  define MTA_HAVE_OPENSSL_init_ssl	1
+#  define MTA_ASN1_STRING_data ASN1_STRING_get0_data
+#  include <openssl/bn.h>
+#  include <openssl/dsa.h>
 # else
 #  define X509_STORE_CTX_get0_cert(ctx)	(ctx)->cert
 #  define MTA_RSA_TMP_CB	1
+#  define MTA_ASN1_STRING_data ASN1_STRING_data
 # endif
 
 # if !TLS_NO_RSA && MTA_RSA_TMP_CB
 static RSA *rsa_tmp = NULL;	/* temporary RSA key */
 static RSA *tmp_rsa_key __P((SSL *, int, int));
-# endif /* !TLS_NO_RSA && MTA_RSA_TMP_CB */
+# endif
 static int	tls_verify_cb __P((X509_STORE_CTX *, void *));
 
 static int x509_verify_cb __P((int, X509_STORE_CTX *));
@@ -74,13 +85,13 @@ static DH *
 get_dh512()
 {
 	DH *dh = NULL;
-#if MTA_HAVE_DH_set0_pqg
+#  if MTA_HAVE_DH_set0_pqg
 	BIGNUM *dhp_bn, *dhg_bn;
-#endif
+#  endif
 
 	if ((dh = DH_new()) == NULL)
 		return NULL;
-#if MTA_HAVE_DH_set0_pqg
+#  if MTA_HAVE_DH_set0_pqg
 	dhp_bn = BN_bin2bn(dh512_p, sizeof (dh512_p), NULL);
 	dhg_bn = BN_bin2bn(dh512_g, sizeof (dh512_g), NULL);
 	if (dhp_bn == NULL || dhg_bn == NULL || !DH_set0_pqg(dh, dhp_bn, NULL, dhg_bn))  {
@@ -89,7 +100,7 @@ get_dh512()
 		BN_free(dhg_bn);
 		return NULL;
 	}
-#else
+#  else
 	dh->p = BN_bin2bn(dh512_p, sizeof(dh512_p), NULL);
 	dh->g = BN_bin2bn(dh512_g, sizeof(dh512_g), NULL);
 	if ((dh->p == NULL) || (dh->g == NULL))
@@ -97,7 +108,7 @@ get_dh512()
 		DH_free(dh);
 		return NULL;
 	}
-#endif
+#  endif
 	return dh;
 }
 
@@ -144,13 +155,13 @@ get_dh2048()
 		};
 	static unsigned char dh2048_g[]={ 0x02, };
 	DH *dh;
-#if MTA_HAVE_DH_set0_pqg
+#  if MTA_HAVE_DH_set0_pqg
 	BIGNUM *dhp_bn, *dhg_bn;
-#endif
+#  endif
 
 	if ((dh=DH_new()) == NULL)
 		return(NULL);
-#if MTA_HAVE_DH_set0_pqg
+#  if MTA_HAVE_DH_set0_pqg
 	dhp_bn = BN_bin2bn(dh2048_p, sizeof (dh2048_p), NULL);
 	dhg_bn = BN_bin2bn(dh2048_g, sizeof (dh2048_g), NULL);
 	if (dhp_bn == NULL || dhg_bn == NULL || !DH_set0_pqg(dh, dhp_bn, NULL, dhg_bn))  {
@@ -159,7 +170,7 @@ get_dh2048()
 		BN_free(dhg_bn);
 		return NULL;
 	}
-#else
+#  else
 	dh->p=BN_bin2bn(dh2048_p,sizeof(dh2048_p),NULL);
 	dh->g=BN_bin2bn(dh2048_g,sizeof(dh2048_g),NULL);
 	if ((dh->p == NULL) || (dh->g == NULL))
@@ -167,7 +178,7 @@ get_dh2048()
 		DH_free(dh);
 		return(NULL);
 	}
-#endif
+#  endif
 	return(dh);
 }
 # endif /* !NO_DH */
@@ -222,8 +233,8 @@ tls_rand_init(randfile, logl)
 	/* set default values */
 	ok = false;
 	done = RI_FAIL;
-	randdef = (randfile == NULL || *randfile == '\0') ? RF_MISS : RF_OK;
-#   if EGD
+	randdef = (SM_IS_EMPTY(randfile)) ? RF_MISS : RF_OK;
+#  if EGD
 	if (randdef == RF_OK && sm_strncasecmp(randfile, "egd:", 4) == 0)
 	{
 		randfile += 4;
@@ -237,7 +248,8 @@ tls_rand_init(randfile, logl)
 			ok = true;
 	}
 	else
-#   endif /* EGD */
+#  endif /* EGD */
+	/* "else" in #if code above */
 	if (randdef == RF_OK && sm_strncasecmp(randfile, "file:", 5) == 0)
 	{
 		int fd;
@@ -378,14 +390,18 @@ init_tls_library(fipsmode)
 {
 	bool bv;
 
+	/*
+	**  OPENSSL_init_ssl(3): "As of version 1.1.0 OpenSSL will
+	**  automatically allocate all resources that it needs
+	**  so no explicit initialisation is required."
+	*/
+
+# if !MTA_HAVE_OPENSSL_init_ssl
 	/* basic TLS initialization, ignore result for now */
 	SSL_library_init();
 	SSL_load_error_strings();
 	OpenSSL_add_all_algorithms();
-# if 0
-	/* this is currently a macro for SSL_library_init */
-	SSLeay_add_ssl_algorithms();
-# endif /* 0 */
+# endif
 
 	bv = true;
 	if (TLSsslidx < 0)
@@ -426,7 +442,7 @@ init_tls_library(fipsmode)
 		if (CertFingerprintAlgorithm == NULL)
 			CertFingerprintAlgorithm = "sha1";
 	}
-#endif /* _FFR_FIPSMODE  */
+# endif /* _FFR_FIPSMODE  */
 
 	if (!TLS_set_engine(SSLEngine, true))
 	{
@@ -487,10 +503,10 @@ tls_set_verify(ctx, ssl, vrfy)
 {
 # if !TLS_VRFY_PER_CTX
 	SSL_set_verify(ssl, vrfy ? SSL_VERIFY_PEER : SSL_VERIFY_NONE, NULL);
-# else /* !TLS_VRFY_PER_CTX */
+# else
 	SSL_CTX_set_verify(ctx, vrfy ? SSL_VERIFY_PEER : SSL_VERIFY_NONE,
 			NULL);
-# endif /* !TLS_VRFY_PER_CTX */
+# endif
 }
 
 /*
@@ -616,6 +632,7 @@ tls_safe_f(var, sff, srv)
 **	Returns:
 **		0/SFF_NORFILES
 */
+
 # define TLS_UNR(bit, req)	(bitset(bit, req) ? SFF_NORFILES : 0)
 # define TLS_OUNR(bit, req)	(bitset(bit, req) ? SFF_NOWRFILES : 0)
 # define TLS_KEYSFF(req)	\
@@ -912,11 +929,11 @@ load_crlpath(ctx, srv, path)
 static char server_session_id_context[] = "sendmail8";
 
 /* 0.9.8a and b have a problem with SSL_OP_TLS_BLOCK_PADDING_BUG */
-#if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
-# define SM_SSL_OP_TLS_BLOCK_PADDING_BUG	1
-#else
-# define SM_SSL_OP_TLS_BLOCK_PADDING_BUG	0
-#endif
+# if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
+#  define SM_SSL_OP_TLS_BLOCK_PADDING_BUG	1
+# else
+#  define SM_SSL_OP_TLS_BLOCK_PADDING_BUG	0
+# endif
 
 bool
 inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhparam)
@@ -937,10 +954,10 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 # if SM_CONF_SHM && !TLS_NO_RSA && MTA_RSA_TMP_CB
 	extern int ShmId;
 # endif
-#if SM_SSL_OP_TLS_BLOCK_PADDING_BUG
+# if SM_SSL_OP_TLS_BLOCK_PADDING_BUG
 	long rt_version;
 	STACK_OF(SSL_COMP) *comp_methods;
-#endif
+# endif
 
 	status = TLS_S_NONE;
 	who = srv ? "server" : "client";
@@ -1017,7 +1034,7 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 	**  /file/name	read parameters from /file/name
 	*/
 
-#define SET_DH_DFL	\
+# define SET_DH_DFL	\
 	do {	\
 		dhparam = "I";	\
 		req |= TLS_I_DHFIXED;	\
@@ -1114,7 +1131,7 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 		return false;
 	}
 
-#if _FFR_VRFY_TRUSTED_FIRST
+# if _FFR_VRFY_TRUSTED_FIRST
 	if (!tTd(88, 101))
 	{
 		X509_STORE *store;
@@ -1124,19 +1141,41 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 		SM_ASSERT(store != NULL);
 		X509_STORE_set_flags(store, X509_V_FLAG_TRUSTED_FIRST);
 	}
-#endif
+# endif
 
 	if (CRLFile != NULL && !load_crlfile(*ctx, srv, CRLFile))
 		return false;
 	if (CRLPath != NULL && !load_crlpath(*ctx, srv, CRLPath))
 		return false;
 
+# if defined(SSL_MODE_AUTO_RETRY) && OPENSSL_VERSION_NUMBER >= 0x10100000L && OPENSSL_VERSION_NUMBER < 0x20000000L
+	/*
+	 *  Turn off blocking I/O handling in OpenSSL: someone turned
+	 *  this on by default in 1.1? should we check first?
+	 */
+#  if _FFR_TESTS
+	if (LogLevel > 9) {
+		sff = SSL_CTX_get_mode(*ctx);
+		if (sff & SSL_MODE_AUTO_RETRY)
+			sm_syslog(LOG_WARNING, NOQID,
+				"STARTTLS=%s, SSL_MODE_AUTO_RETRY=set, mode=%#lx",
+				who, sff);
+	}
+
+	/* hack for testing! */
+	if (tTd(96, 101) || getenv("SSL_MODE_AUTO_RETRY") != NULL)
+			SSL_CTX_set_mode(*ctx, SSL_MODE_AUTO_RETRY);
+	else
+#  endif /* _FFR_TESTS */
+	/* "else" in #if code above */
+	SSL_CTX_clear_mode(*ctx, SSL_MODE_AUTO_RETRY);
+# endif /* defined(SSL_MODE_AUTO_RETRY) && OPENSSL_VERSION_NUMBER >= 0x10100000L && OPENSSL_VERSION_NUMBER < 0x20000000L */
+
 # if TLS_NO_RSA
 	/* turn off backward compatibility, required for no-rsa */
 	SSL_CTX_set_options(*ctx, SSL_OP_NO_SSLv2);
 # endif
 
-
 # if !TLS_NO_RSA && MTA_RSA_TMP_CB
 	/*
 	**  Create a temporary RSA key
@@ -1187,15 +1226,15 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 			return false;
 	}
 
-#if _FFR_TLS_USE_CERTIFICATE_CHAIN_FILE
-# define SSL_CTX_use_cert(ssl_ctx, certfile) \
+# if _FFR_TLS_USE_CERTIFICATE_CHAIN_FILE
+#  define SSL_CTX_use_cert(ssl_ctx, certfile) \
 	SSL_CTX_use_certificate_chain_file(ssl_ctx, certfile)
-# define SSL_CTX_USE_CERT "SSL_CTX_use_certificate_chain_file"
-#else
-# define SSL_CTX_use_cert(ssl_ctx, certfile) \
+#  define SSL_CTX_USE_CERT "SSL_CTX_use_certificate_chain_file"
+# else
+#  define SSL_CTX_use_cert(ssl_ctx, certfile) \
 	SSL_CTX_use_certificate_file(ssl_ctx, certfile, SSL_FILETYPE_PEM)
-# define SSL_CTX_USE_CERT "SSL_CTX_use_certificate_file"
-#endif
+#  define SSL_CTX_USE_CERT "SSL_CTX_use_certificate_file"
+# endif
 
 	/* get the certificate file */
 	if (bitset(TLS_S_CERT_OK, status) &&
@@ -1272,7 +1311,7 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 
 	/* SSL_CTX_set_quiet_shutdown(*ctx, 1); violation of standard? */
 
-#if SM_SSL_OP_TLS_BLOCK_PADDING_BUG
+# if SM_SSL_OP_TLS_BLOCK_PADDING_BUG
 
 	/*
 	**  In OpenSSL 0.9.8[ab], enabling zlib compression breaks the
@@ -1284,21 +1323,21 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 	**  just the compile time version.
 	*/
 
-	rt_version = SSLeay();
+	rt_version = TLS_version_num();
 	if (rt_version >= 0x00908000L && rt_version <= 0x0090802fL)
 	{
 		comp_methods = SSL_COMP_get_compression_methods();
 		if (comp_methods != NULL && sk_SSL_COMP_num(comp_methods) > 0)
 			options &= ~SSL_OP_TLS_BLOCK_PADDING_BUG;
 	}
-#endif
+# endif
 	SSL_CTX_set_options(*ctx, (long) options);
 
 # if !NO_DH
 	/* Diffie-Hellman initialization */
 	if (bitset(TLS_I_TRY_DH, req))
 	{
-#  if TLS_EC
+#  if TLS_EC == 1
 		EC_KEY *ecdh;
 #  endif
 
@@ -1346,7 +1385,7 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 			if (tTd(96, 2))
 				sm_dprintf("inittls: Generating %d bit DH parameters\n", bits);
 
-#if MTA_HAVE_DSA_GENERATE_EX
+#  if MTA_HAVE_DSA_GENERATE_EX
 			dsa = DSA_new();
 			if (dsa != NULL)
 			{
@@ -1355,12 +1394,12 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 				if (r != 0)
 					dh = DSA_dup_DH(dsa);
 			}
-#else
+#  else
 			/* this takes a while! */
 			dsa = DSA_generate_parameters(bits, NULL, 0, NULL,
 						      NULL, 0, NULL);
 			dh = DSA_dup_DH(dsa);
-#endif
+#  endif
 			DSA_free(dsa);
 		}
 		else if (dh == NULL && bitset(TLS_I_DHFIXED, req))
@@ -1404,7 +1443,10 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 			DH_free(dh);
 		}
 
-#  if TLS_EC
+#  if TLS_EC == 2
+		SSL_CTX_set_options(*ctx, SSL_OP_SINGLE_ECDH_USE);
+		SSL_CTX_set_ecdh_auto(*ctx, 1);
+#  elif TLS_EC == 1
 		ecdh = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
 		if (ecdh != NULL)
 		{
@@ -1466,22 +1508,19 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 			**  but we hope that that function will later on
 			**  only set the mode per connection.
 			*/
+
 			SSL_CTX_set_verify(*ctx,
 				bitset(TLS_I_NO_VRFY, req) ? SSL_VERIFY_NONE
 							   : SSL_VERIFY_PEER,
 				NULL);
 
-			/* install verify callback */
-# if _FFR_TLSA_DANE
-			Dane_vrfy_ctx.dane_vrfy_chk = Dane;
-			SSL_CTX_set_cert_verify_callback(*ctx, tls_verify_cb,
-				(void *)&Dane_vrfy_ctx);
-# else
+			if (srv)
+			{
+				SSL_CTX_set_client_CA_list(*ctx,
+					SSL_load_client_CA_file(cacertfile));
+			}
 			SSL_CTX_set_cert_verify_callback(*ctx, tls_verify_cb,
 							NULL);
-# endif
-			SSL_CTX_set_client_CA_list(*ctx,
-				SSL_load_client_CA_file(cacertfile));
 		}
 		else
 		{
@@ -1491,12 +1530,15 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 			**  which in turn would be necessary
 			**  if we want to allow relaying based on it.
 			*/
+
 			if (LogLevel > 5)
 			{
 				sm_syslog(LOG_WARNING, NOQID,
 					  "STARTTLS=%s, error: load verify locs %s, %s failed: %d",
 					  who, cacertpath, cacertfile, r);
-				tlslogerr(LOG_WARNING, 9, who);
+				tlslogerr(LOG_WARNING,
+					bitset(TLS_I_VRFY_LOC, req) ? 8 : 9,
+					who);
 			}
 			if (bitset(TLS_I_VRFY_LOC, req))
 				return false;
@@ -1524,6 +1566,25 @@ inittls(ctx, req, options, srv, certfile, keyfile, cacertpath, cacertfile, dhpar
 		}
 	}
 
+# if MTA_HAVE_TLSv1_3
+	/* install our own cipher suites */
+	if (!SM_IS_EMPTY(CipherSuites))
+	{
+		if (SSL_CTX_set_ciphersuites(*ctx, CipherSuites) <= 0)
+		{
+			if (LogLevel > 7)
+			{
+				sm_syslog(LOG_WARNING, NOQID,
+					  "STARTTLS=%s, error: SSL_CTX_set_ciphersuites(%s) failed, suites ignored",
+					  who, CipherSuites);
+
+				tlslogerr(LOG_WARNING, 9, who);
+			}
+			/* failure if setting to this suites is required? */
+		}
+	}
+# endif /* MTA_HAVE_TLSv1_3 */
+
 	if (LogLevel > 12)
 		sm_syslog(LOG_INFO, NOQID, "STARTTLS=%s, init=%d", who, ok);
 
@@ -1634,13 +1695,15 @@ static void
 getaltnames(cert, srv, host)
 	X509 *cert;
 	bool srv;
-	char *host;
+	const char *host;
 {
 	STACK_OF(GENERAL_NAME) *gens;
 	int i, j, len, r;
 	const GENERAL_NAME *gn;
 	char *dnsname, *who;
 
+	if (!SetCertAltnames)
+		return;
 	who = srv ? "server" : "client";
 	gens = X509_get_ext_d2i(cert, NID_subject_alt_name, 0, 0);
 	if (gens == NULL)
@@ -1662,7 +1725,7 @@ getaltnames(cert, srv, host)
 					who, whichhost);
 			continue;
 		}
-		dnsname = (char *) ASN1_STRING_data(gn->d.ia5);
+		dnsname = (char *) MTA_ASN1_STRING_data(gn->d.ia5);
 		if (dnsname == NULL)
 			continue;
 		len = ASN1_STRING_length(gn->d.ia5);
@@ -1725,6 +1788,9 @@ tls_get_info(ssl, srv, host, mac, certreq)
 	char *s, *who;
 	char bitstr[16];
 	X509 *cert;
+# if DANE
+	dane_vrfy_ctx_P dane_vrfy_ctx;
+# endif
 
 	c = SSL_get_current_cipher(ssl);
 
@@ -1754,7 +1820,7 @@ tls_get_info(ssl, srv, host, mac, certreq)
 	if (cert != NULL)
 	{
 		X509_NAME *subj, *issuer;
-		char buf[MAXNAME];
+		char buf[MAXNAME];	/* EAI: not affected */
 
 		subj = X509_get_subject_name(cert);
 		issuer = X509_get_issuer_name(cert);
@@ -1819,19 +1885,31 @@ tls_get_info(ssl, srv, host, mac, certreq)
 		macdefine(mac, A_PERM, macid("{cn_issuer}"), "");
 		macdefine(mac, A_TEMP, macid(CERTFPMACRO), "");
 	}
-# if _FFR_TLSA_DANE
-	if (Dane_vrfy_ctx.dane_vrfy_res == DANE_VRFY_OK)
+# if DANE
+	dane_vrfy_ctx = NULL;
+	if (TLSsslidx >= 0)
+	{
+		tlsi_ctx_T *tlsi_ctx;
+
+		tlsi_ctx = (tlsi_ctx_P) SSL_get_ex_data(ssl, TLSsslidx);
+		if (tlsi_ctx != NULL)
+			dane_vrfy_ctx = &(tlsi_ctx->tlsi_dvc);
+	}
+#  define DANE_VRFY_RES_IS(r) \
+	((dane_vrfy_ctx != NULL) && dane_vrfy_ctx->dane_vrfy_res == (r))
+	if (DANE_VRFY_RES_IS(DANE_VRFY_OK))
 	{
 		s = "TRUSTED";
 		r = TLS_AUTH_OK;
 	}
-	else if (Dane_vrfy_ctx.dane_vrfy_res == DANE_VRFY_FAIL)
+	else if (DANE_VRFY_RES_IS(DANE_VRFY_FAIL))
 	{
 		s = "DANE_FAIL";
 		r = TLS_AUTH_FAIL;
 	}
 	else
-# endif
+# endif /* if DANE */
+	/* "else" in #if code above */
 	switch (verifyok)
 	{
 	  case X509_V_OK:
@@ -1866,14 +1944,25 @@ tls_get_info(ssl, srv, host, mac, certreq)
 		s1 = macget(mac, macid("{verify}"));
 		s2 = macget(mac, macid("{cipher}"));
 
+# if DANE
+#  define LOG_DANE_FP	\
+	('\0' != dane_vrfy_ctx->dane_vrfy_fp[0] && DANE_VRFY_RES_IS(DANE_VRFY_FAIL))
+# endif
 		/* XXX: maybe cut off ident info? */
 		sm_syslog(LOG_INFO, NOQID,
-			  "STARTTLS=%s, relay=%.100s, version=%.16s, verify=%.16s, cipher=%.64s, bits=%.6s/%.6s",
+			  "STARTTLS=%s, relay=%.100s, version=%.16s, verify=%.16s, cipher=%.64s, bits=%.6s/%.6s%s%s",
 			  who,
 			  host == NULL ? "local" : host,
 			  vers, s1, s2, /* sm_snprintf() can deal with NULL */
 			  algbits == NULL ? "0" : algbits,
-			  cbits == NULL ? "0" : cbits);
+			  cbits == NULL ? "0" : cbits
+# if DANE
+			, LOG_DANE_FP ? ", pubkey_fp=" : ""
+			, LOG_DANE_FP ? dane_vrfy_ctx->dane_vrfy_fp : ""
+# else
+			, "", ""
+# endif
+			);
 		if (LogLevel > 11)
 		{
 			/*
@@ -1995,9 +2084,9 @@ endtls(pssl, who)
 **		temporary RSA key.
 */
 
-#   ifndef MAX_RSA_TMP_CNT
-#    define MAX_RSA_TMP_CNT	1000	/* XXX better value? */
-#   endif
+#  ifndef MAX_RSA_TMP_CNT
+#   define MAX_RSA_TMP_CNT	1000	/* XXX better value? */
+#  endif
 
 /* ARGUSED0 */
 static RSA *
@@ -2006,14 +2095,14 @@ tmp_rsa_key(s, export, keylength)
 	int export;
 	int keylength;
 {
-#   if SM_CONF_SHM
+#  if SM_CONF_SHM
 	extern int ShmId;
 	extern int *PRSATmpCnt;
 
 	if (ShmId != SM_SHM_NO_ID && rsa_tmp != NULL &&
 	    ++(*PRSATmpCnt) < MAX_RSA_TMP_CNT)
 		return rsa_tmp;
-#   endif /* SM_CONF_SHM */
+#  endif /* SM_CONF_SHM */
 
 	if (rsa_tmp != NULL)
 		RSA_free(rsa_tmp);
@@ -2026,8 +2115,8 @@ tmp_rsa_key(s, export, keylength)
 	}
 	else
 	{
-#   if SM_CONF_SHM
-#    if 0
+#  if SM_CONF_SHM
+#   if 0
 		/*
 		**  XXX we can't (yet) share the new key...
 		**	The RSA structure contains pointers hence it can't be
@@ -2038,8 +2127,8 @@ tmp_rsa_key(s, export, keylength)
 
 		if (ShmId != SM_SHM_NO_ID)
 			*PRSATmpCnt = 0;
-#    endif /* 0 */
-#   endif /* SM_CONF_SHM */
+#   endif /* 0 */
+#  endif /* SM_CONF_SHM */
 		if (LogLevel > 9)
 			sm_syslog(LOG_ERR, NOQID,
 				  "STARTTLS=server, tmp_rsa_key: new temp RSA key");
@@ -2160,7 +2249,7 @@ tls_verify_log(ok, ctx, name)
 
 #define SM_DECTLSI	\
 	tlsi_ctx_T *tlsi_ctx;	\
-	SSL *ssl;
+	SSL *ssl
 #define SM_GETTLSI	\
 	do {		\
 		tlsi_ctx = NULL;	\
@@ -2175,101 +2264,175 @@ tls_verify_log(ok, ctx, name)
 	while (0)
 
 
-# if _FFR_TLSA_DANE
+# if DANE
+
+/*
+**  DANE_GET_TLSA -- Retrieve TLSA RR for DANE
+**
+**	Parameters:
+**		dane -- dane verify context
+**
+**	Returns:
+**		dane_tlsa if TLSA RR is available
+**		NULL otherwise
+*/
+
+dane_tlsa_P
+dane_get_tlsa(dane_vrfy_ctx)
+	dane_vrfy_ctx_P dane_vrfy_ctx;
+{
+	STAB *s;
+	dane_tlsa_P dane_tlsa;
+
+	dane_tlsa = NULL;
+	if (NULL == dane_vrfy_ctx)
+		return NULL;
+	if (dane_vrfy_ctx->dane_vrfy_chk == DANE_NEVER ||
+	    dane_vrfy_ctx->dane_vrfy_host == NULL)
+		return NULL;
+
+	GETTLSANOX(dane_vrfy_ctx->dane_vrfy_host, &s,
+		dane_vrfy_ctx->dane_vrfy_port);
+	if (NULL == s)
+		goto notfound;
+	dane_tlsa = s->s_tlsa;
+	if (NULL == dane_tlsa)
+		goto notfound;
+	if (0 == dane_tlsa->dane_tlsa_n)
+		goto notfound;
+	if (tTd(96, 4))
+		sm_dprintf("dane_get_tlsa, chk=%d, host=%s, n=%d, stat=entry found\n",
+			dane_vrfy_ctx->dane_vrfy_chk,
+			dane_vrfy_ctx->dane_vrfy_host, dane_tlsa->dane_tlsa_n);
+	return dane_tlsa;
+
+  notfound:
+	if (tTd(96, 4))
+		sm_dprintf("dane_get_tlsa, chk=%d, host=%s, stat=no valid entry found\n",
+			dane_vrfy_ctx->dane_vrfy_chk,
+			dane_vrfy_ctx->dane_vrfy_host);
+	return NULL;
+}
 
 /*
 **  DANE_VERIFY -- verify callback for TLS certificates
 **
 **	Parameters:
 **		ctx -- X509 context
-**		dane -- callback context
+**		dane_vrfy_ctx -- callback context
 **
 **	Returns:
 **		DANE_VRFY_{OK,NONE,FAIL}
 */
 
+/* NOTE: this only works because the "matching type" is 0, 1, 2 for these! */
+static const char *dane_mdalgs[] = { "", "sha256", "sha512" };
+
 static int
 dane_verify(ctx, dane_vrfy_ctx)
 	X509_STORE_CTX *ctx;
 	dane_vrfy_ctx_P dane_vrfy_ctx;
 {
-	int r, i, ok;
-	STAB *s;
+	int r, i, ok, mdalg;
 	X509 *cert;
 	dane_tlsa_P dane_tlsa;
-	const EVP_MD *md;
 	char *fp;
 
-	if (dane_vrfy_ctx == NULL)
-		return DANE_VRFY_NONE;
-	dane_vrfy_ctx->dane_vrfy_res = DANE_VRFY_NONE;
-	if (dane_vrfy_ctx->dane_vrfy_chk == DANE_NEVER ||
-	    dane_vrfy_ctx->dane_vrfy_host == NULL)
-	{
-		return DANE_VRFY_NONE;
-	}
-
-	s = stab(dane_vrfy_ctx->dane_vrfy_host, ST_TLSA_RR, ST_FIND);
-	if (tTd(96, 4))
-		sm_dprintf("dane_verify, host=%s, s=%p\n",
-			dane_vrfy_ctx->dane_vrfy_host, s);
-	if (s == NULL)
-		return DANE_VRFY_NONE;
-	dane_tlsa = s->s_tlsa;
+	dane_tlsa = dane_get_tlsa(dane_vrfy_ctx);
 	if (dane_tlsa == NULL)
 		return DANE_VRFY_NONE;
-	if (dane_tlsa->dane_tlsa_n == 0)
-		return DANE_VRFY_NONE;
 
+	dane_vrfy_ctx->dane_vrfy_fp[0] = '\0';
 	cert = X509_STORE_CTX_get0_cert(ctx);
-	if (tTd(96, 4))
-		sm_dprintf("dane_verify, cert=%p\n", cert);
+	if (tTd(96, 8))
+		sm_dprintf("dane_verify, cert=%p\n", (void *)cert);
 	if (cert == NULL)
 		return DANE_VRFY_FAIL;
 
-	md = EVP_get_digestbyname("sha256");
-	if (NULL == md)
-		return DANE_VRFY_FAIL;
 	ok = DANE_VRFY_NONE;
-	for (i = 0; i < dane_tlsa->dane_tlsa_n; i++)
+	fp = NULL;
+
+	/*
+	**  If the TLSA RRs would be sorted the two loops below could
+	**  be merged into one and simply change mdalg when it changes
+	**  in dane_tlsa->dane_tlsa_rr.
+	*/
+
+	/* use a different order? */
+	for (mdalg = 0; mdalg < SM_ARRAY_SIZE(dane_mdalgs); mdalg++)
 	{
-		char *p;
-
-		p = dane_tlsa->dane_tlsa_rr[i];
-
-		/* ignore bogus/unsupported TLSA RRs */
-		if (dane_tlsa_chk(p, dane_tlsa->dane_tlsa_len[i],
-				  dane_vrfy_ctx->dane_vrfy_host, false)
-		    != TLSA_OK)
-			continue;
-
-		r = pubkey_fp(cert, md, &fp);
-		if (tTd(96, 4))
-			sm_dprintf("dane_verify, r=%d, len=%d\n", r, dane_tlsa->dane_tlsa_len[i]);
-		if (r != dane_tlsa->dane_tlsa_len[i] - 3)
-			continue;
-		ok = DANE_VRFY_FAIL;
-		if (memcmp(p + 3, fp, r) == 0)
+		SM_FREE(fp);
+		r = 0;
+		for (i = 0; i < dane_tlsa->dane_tlsa_n; i++)
 		{
-			if (tTd(96, 2))
-				sm_dprintf("dane_verify, status=match\n");
-			if (tTd(96, 8))
-			{
-				unsigned char hex[256];
+			char *p;
+			int alg;
 
-				data2hex((unsigned char *)p,
-					dane_tlsa->dane_tlsa_len[i], hex,
-					sizeof(hex));
-				sm_dprintf("dane_verify, pubkey_fp=%s\n", hex);
+			p = dane_tlsa->dane_tlsa_rr[i];
+
+			/* ignore bogus/unsupported TLSA RRs */
+			alg = dane_tlsa_chk(p, dane_tlsa->dane_tlsa_len[i],
+					  dane_vrfy_ctx->dane_vrfy_host, false);
+			if (tTd(96, 8))
+				sm_dprintf("dane_verify, alg=%d, mdalg=%d\n",
+					alg, mdalg);
+			if (alg != mdalg)
+				continue;
+
+			if (NULL == fp)
+			{
+				r = pubkey_fp(cert, dane_mdalgs[mdalg], &fp);
+				if (NULL == fp)
+					return DANE_VRFY_FAIL;
+					/* or continue? */
+			}
+
+			/* just for logging */
+			if (r > 0 && fp != NULL)
+			{
+				(void) data2hex((unsigned char *)fp, r,
+					(unsigned char *)dane_vrfy_ctx->dane_vrfy_fp,
+					sizeof(dane_vrfy_ctx->dane_vrfy_fp));
+			}
+
+			if (tTd(96, 4))
+				sm_dprintf("dane_verify, alg=%d, r=%d, len=%d\n",
+					alg, r, dane_tlsa->dane_tlsa_len[i]);
+			if (r != dane_tlsa->dane_tlsa_len[i] - 3)
+				continue;
+			ok = DANE_VRFY_FAIL;
+
+			/*
+			**  Note: Type is NOT checked because only 3-1-x
+			**  is supported.
+			*/
+
+			if (memcmp(p + 3, fp, r) == 0)
+			{
+				if (tTd(96, 2))
+					sm_dprintf("dane_verify, status=match\n");
+				if (tTd(96, 8))
+				{
+					unsigned char hex[256];
+
+					data2hex((unsigned char *)p,
+						dane_tlsa->dane_tlsa_len[i],
+						hex, sizeof(hex));
+					sm_dprintf("dane_verify, pubkey_fp=%s\n"
+						, hex);
+				}
+				dane_vrfy_ctx->dane_vrfy_res = DANE_VRFY_OK;
+				SM_FREE(fp);
+				return DANE_VRFY_OK;
 			}
-			dane_vrfy_ctx->dane_vrfy_res = DANE_VRFY_OK;
-			return DANE_VRFY_OK;
 		}
 	}
+
+	SM_FREE(fp);
 	dane_vrfy_ctx->dane_vrfy_res = ok;
 	return ok;
 }
-# endif
+# endif /* DANE */
 
 /*
 **  TLS_VERIFY_CB -- verify callback for TLS certificates
@@ -2289,21 +2452,31 @@ tls_verify_cb(ctx, cb_ctx)
 	void *cb_ctx;
 {
 	int ok;
+# if DANE
+	SM_DECTLSI;
+# endif
 
 	/*
-	**  man SSL_CTX_set_cert_verify_callback():
+	**  SSL_CTX_set_cert_verify_callback(3):
 	**  callback should return 1 to indicate verification success
 	**  and 0 to indicate verification failure.
 	*/
 
-# if _FFR_TLSA_DANE
-	ok = dane_verify(ctx, (dane_vrfy_ctx_P) cb_ctx);
-	if (tTd(96, 2))
-		sm_dprintf("dane_verify=%d, res=%d\n", ok,
-			Dane_vrfy_ctx.dane_vrfy_res);
-	if (ok != DANE_VRFY_NONE)
-		return 1;
-# endif
+# if DANE
+	SM_GETTLSI;
+	if (tlsi_ctx != NULL)
+	{
+		dane_vrfy_ctx_P dane_vrfy_ctx;
+
+		dane_vrfy_ctx = &(tlsi_ctx->tlsi_dvc);
+		ok = dane_verify(ctx, dane_vrfy_ctx);
+		if (tTd(96, 2))
+			sm_dprintf("dane_verify=%d, res=%d\n", ok,
+				dane_vrfy_ctx->dane_vrfy_res);
+		if (ok != DANE_VRFY_NONE)
+			return 1;
+	}
+# endif /* DANE */
 
 	ok = X509_verify_cert(ctx);
 	if (ok <= 0)
@@ -2336,19 +2509,17 @@ tlslogerr(priority, ll, who)
 {
 	unsigned long l;
 	int line, flags;
-	unsigned long es;
 	char *file, *data;
 	char buf[256];
 
 	if (LogLevel <= ll)
 		return;
-	es = CRYPTO_thread_id();
 	while ((l = ERR_get_error_line_data((const char **) &file, &line,
 					    (const char **) &data, &flags))
 		!= 0)
 	{
 		sm_syslog(priority, NOQID,
-			  "STARTTLS=%s: %lu:%s:%s:%d:%s", who, es,
+			  "STARTTLS=%s: %s:%s:%d:%s", who,
 			  ERR_error_string(l, buf),
 			  file, line,
 			  bitset(ERR_TXT_STRING, flags) ? data : "");
@@ -2372,7 +2543,7 @@ x509_verify_cb(ok, ctx)
 	int ok;
 	X509_STORE_CTX *ctx;
 {
-	SM_DECTLSI
+	SM_DECTLSI;
 
 	if (ok != 0)
 		return ok;
@@ -2389,7 +2560,7 @@ x509_verify_cb(ok, ctx)
 	return ok;
 }
 
-# if !USE_OPENSSL_ENGINE
+# if !USE_OPENSSL_ENGINE && !defined(OPENSSL_NO_ENGINE)
 /*
 **  TLS_SET_ENGINE -- set up ENGINE if needed
 **
@@ -2422,8 +2593,14 @@ TLS_set_engine(id, isprefork)
 			TLSEngineInitialized);
 	if (TLSEngineInitialized)
 		return 1;
-	if (id == NULL || *id == '\0')
+	if (SM_IS_EMPTY(id))
 		return 1;
+#  if !defined(ENGINE_METHOD_ALL)
+	if (LogLevel > 9)
+		sm_syslog(LOG_NOTICE, NOQID,
+			"engine=%s, status=engines_not_support", id)
+	goto error;
+#  endif
 
 	/* is this the "right time" to initialize the engine? */
 	if (isprefork != SSLEngineprefork)
@@ -2490,10 +2667,10 @@ TLS_set_engine(id, isprefork)
 				"engine=%s, set_default=failed", id);
 		goto error;
 	}
-#ifdef ENGINE_CTRL_CHIL_SET_FORKCHECK
+#  ifdef ENGINE_CTRL_CHIL_SET_FORKCHECK
 	if (strcmp(id, "chil") == 0)
 		ENGINE_ctrl(e, ENGINE_CTRL_CHIL_SET_FORKCHECK, 1, 0, 0);
-#endif
+#  endif
 
 	/* Free our "structural" reference. */
 	ENGINE_free(e);
@@ -2508,5 +2685,5 @@ TLS_set_engine(id, isprefork)
 		ENGINE_free(e);
 	return 0;
 }
-# endif /* !USE_OPENSSL_ENGINE */
+# endif /* !USE_OPENSSL_ENGINE && !defined(OPENSSL_NO_ENGINE) */
 #endif /* STARTTLS */
