@@ -28,16 +28,14 @@ SM_IDSTR(id, "@(#)$Id: vacation.c,v 8.148 2013-11-22 20:52:02 ca Exp $")
 #include <syslog.h>
 #include <time.h>
 #include <unistd.h>
-#ifdef EX_OK
-# undef EX_OK		/* unistd.h may have another use for this */
-#endif /* EX_OK */
+#include <sm/sendmail.h>
 #include <sm/sysexits.h>
 
 #include <sm/cf.h>
 #include <sm/mbdb.h>
-#include "sendmail/sendmail.h"
+#include <sendmail/sendmail.h>
 #include <sendmail/pathnames.h>
-#include "libsmdb/smdb.h"
+#include <libsmdb/smdb.h>
 
 #define ONLY_ONCE	((time_t) 0)	/* send at most one reply */
 #define INTERVAL_UNDEF	((time_t) (-1))	/* no value given */
@@ -93,16 +91,16 @@ bool CloseMBDB = false;
 #if defined(__hpux) || defined(__osf__)
 # ifndef SM_CONF_SYSLOG_INT
 #  define SM_CONF_SYSLOG_INT	1
-# endif /* SM_CONF_SYSLOG_INT */
+# endif
 #endif /* defined(__hpux) || defined(__osf__) */
 
 #if SM_CONF_SYSLOG_INT
 # define SYSLOG_RET_T	int
 # define SYSLOG_RET	return 0
-#else /* SM_CONF_SYSLOG_INT */
+#else
 # define SYSLOG_RET_T	void
 # define SYSLOG_RET
-#endif /* SM_CONF_SYSLOG_INT */
+#endif
 
 typedef SYSLOG_RET_T SYSLOG_T __P((int, const char *, ...));
 SYSLOG_T *msglog = syslog;
@@ -176,11 +174,11 @@ main(argc, argv)
 			    "Unknown UID %d", (int) RealUid);
 	RunAsUserName = RealUserName = rnamebuf;
 
-# ifdef LOG_MAIL
+#ifdef LOG_MAIL
 	openlog("vacation", LOG_PID, LOG_MAIL);
-# else /* LOG_MAIL */
+#else
 	openlog("vacation", LOG_PID);
-# endif /* LOG_MAIL */
+#endif
 
 	opterr = 0;
 	initdb = false;
@@ -650,8 +648,7 @@ nsearch(name, str)
 		**  matching "eric" to "<eric+detail>".
 		*/
 
-		if (tolower(*s) == tolower(*name) &&
-		    strncasecmp(name, s, len) == 0 &&
+		if (SM_STRNCASEEQ(name, s, len) &&
 		    (s == str || !isascii(*(s - 1)) || !isalnum(*(s - 1))) &&
 		    (!isascii(*(s + len)) || !isalnum(*(s + len))))
 			return true;
@@ -767,7 +764,7 @@ junkmail(from)
 #if 0
 	if (quot)
 		return false;	/* syntax error... */
-#endif /* 0 */
+#endif
 
 	/* test prefixes */
 	for (cur = ignorepre; cur->name != NULL; ++cur)
@@ -1041,9 +1038,9 @@ sendmessage(myname, msgfn, sender)
 #if _FFR_VAC_WAIT4SM
 # ifdef WAITUNION
 		union wait st;
-# else /* WAITUNION */
+# else
 		auto int st;
-# endif /* WAITUNION */
+# endif
 #endif /* _FFR_VAC_WAIT4SM */
 
 		(void) sm_io_fprintf(sfp, SM_TIME_DEFAULT, "To: %s\n", From);
@@ -1055,7 +1052,7 @@ sendmessage(myname, msgfn, sender)
 		(void) sm_io_close(sfp, SM_TIME_DEFAULT);
 #if _FFR_VAC_WAIT4SM
 		(void) wait(&st);
-#endif /* _FFR_VAC_WAIT4SM */
+#endif
 	}
 	else
 	{
